/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal1type.h"
#include "deal1ext.h"
/******************************************************************************/
/*                   Initialize simulation variables.
*******************************************************************************/
int Sim_Init()
{
 Map_Init();
 Pre_Amble_Init();
 Post_Amb_Init();
 Noise_Var_Init();
 Noise_Gen_Init();
 LO_Init();
 Ber_Init();
 return(0);
}
/******************************************************************************/
/*                               BPSK mapping.
*******************************************************************************/
int Map_Init()
{
 Map[0]= 1.0;
 Map[1]=-1.0;
 return(0);
}
/******************************************************************************/
/*                       Initialize the preamble.
*******************************************************************************/
int Pre_Amble_Init()
{
 int cnt,bit0,bit1;
 long int random(void);
 printf("\t Preamble:\n");
 for(cnt=ZERO;cnt<PRE_AMB_SIZE;cnt++)
 {
  bit0=random()&ONE;
  bit1=random()&ONE;
  Sym[cnt]=TWO*bit1+bit0;
  printf("\t\t Time:%d re:%d im:%d\n",cnt,bit0,bit1);
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                            Initialize th postamble.
*******************************************************************************/
int Post_Amb_Init()
{
 int cnt,cnt1,bit0,bit1;
 long int random(void);
 cnt1=PRE_AMB_SIZE+SYM_SIZE;;
 for(cnt=ZERO;cnt<POST_AMBLE;cnt++)
 {
  bit0=random()&ONE;
  bit1=random()&ONE;
  if(cnt1 == FRAME_SIZE)
   Error_Mes(3);
  Sym[cnt1]=TWO*bit1+bit0;
  cnt1++;
 }
 return(0);
}
/******************************************************************************/
/*                   Initialize the transmit filter register.
*******************************************************************************/
int Tx_Reg_Init()
{
 int cnt;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Re_Tx_Reg[cnt]=Im_Tx_Reg[cnt]=0.0;
 return(0);
}
/******************************************************************************/
/*                        Initialize the noise variance.
*******************************************************************************/
int Noise_Var_Init()
{
 if(SNR_B < MAX_SNR)
  Variance=0.5/pow(10.0,0.1*SNR_B);
 else
  Variance=0.0;
 printf("\t SNR per bit:%lf Noise variance:%lf.\n",SNR_B,Variance);
 Variance=0.5*Variance;                /* Noise variance for passband signal. */
 printf("\t Variance of noise added to passband signal:%lf.\n\n",Variance);
 return(0);
}
/******************************************************************************/
/*                     Initialize the noise generator.
*******************************************************************************/
int Noise_Gen_Init()
{
 int cnt;
 long int random(void);
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
 {
  N1_Store[cnt]=(unsigned short)random();
  N2_Store[cnt]=(unsigned short)random();
 }
 return(0);
}
/******************************************************************************/
/*                           Initialize the LO.
*******************************************************************************/
int LO_Init()
{
 int cnt;
 double theta1;
 printf("\t Local oscillator values:\n");
 for(cnt=ZERO;cnt<LO_PERIOD;cnt++)
 {
  theta1=2.0*M_PI*(double)cnt/(double)LO_PERIOD;
  Re_LO[cnt]=  rint(2.0*cos(theta1));
  Im_LO[cnt]= -rint(2.0*sin(theta1));
  printf("\t\t Time:%d cos:%+lf sin:%+lf\n",cnt,Re_LO[cnt],Im_LO[cnt]);
 }
 Phase_Var=0.0;
 Tim_Var=0.0;
 return(0);
}
/******************************************************************************/
/*                           Initialize the BER.
*******************************************************************************/
int Ber_Init()
{
 Ber=ZERO;
 return(0);
}
/******************************************************************************/
