/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal1type.h"
#include "deal1ext.h"
/******************************************************************************/
int Get_Sync()
{
 int cnt1;
 Init_Sync();
 for(cnt1=ZERO;cnt1<RX_SAMP_SIZE;cnt1++)
 {
  Compute_Tim_Correlation(cnt1);
  Check_Tim_Correlation(cnt1);
 }
 if(!RAND_TIM)
  Compute_Tim_Var();
 Compute_Carrier_Phase();
 Compute_Amplitude();
 Compute_Variance();
 Print_Sync_Results();
 return(0);
}
/******************************************************************************/
/*                 Initialize the synchronization variables.
*******************************************************************************/
int Init_Sync()
{
 Tim_Max_Mag=MIN_MAG;
 Re_Mean_Corr=Im_Mean_Corr=0.0;
 Mean_Corr_Cnt=ZERO;
 return(0);
}
/******************************************************************************/
/*                      Compute the current correlation.
*******************************************************************************/
int Compute_Tim_Correlation(cnt1)
int cnt1;
{
 int cnt2,cnt3,bit0,bit1;
 double re_sym,im_sym;
 cnt2=cnt1;
 cnt3=ZERO;
 Re_Tim_Corr=Im_Tim_Corr=0.0;
 while((cnt2 < RX_SAMP_SIZE) && (cnt3 < PRE_AMB_SIZE))
 {
  bit0=Sym[cnt3]&ONE;
  bit1=Sym[cnt3]>>ONE;
  re_sym=Map[bit0];
  im_sym=Map[bit1];
  Re_Tim_Corr=Re_Tim_Corr+Re_MF_Op[cnt2]*re_sym+Im_MF_Op[cnt2]*im_sym;
  Im_Tim_Corr=Im_Tim_Corr+Im_MF_Op[cnt2]*re_sym-Re_MF_Op[cnt2]*im_sym;
  cnt2=cnt2+RX_SAMP_PER_SYM;
  cnt3++;
 }
 Re_Mean_Corr=Re_Mean_Corr+Re_Tim_Corr;
 Im_Mean_Corr=Im_Mean_Corr+Im_Tim_Corr;
 Mean_Corr_Cnt++;
 return(0);
}
/******************************************************************************/
/*                 Check the timing correlation for maximum.
*******************************************************************************/
int Check_Tim_Correlation(cnt1)
int cnt1;
{
 double mag;
 mag=Re_Tim_Corr*Re_Tim_Corr+Im_Tim_Corr*Im_Tim_Corr;
 if(mag > Tim_Max_Mag)
 {
  Tim_Max_Mag=mag;
  Re_Tim_Max=Re_Tim_Corr;
  Im_Tim_Max=Im_Tim_Corr;
  Samp_Instant=cnt1;
 }
 if(Frame_Cnt == ZERO)
  fprintf(fp1,"%lf\n",mag);
 return(0);
}
/******************************************************************************/
/*                         Compute the timing variance.
*******************************************************************************/
int Compute_Tim_Var()
{
 double correct_time,est_time;
 correct_time=(double)(TX_DELAY*RX_IP_FAC*TWO);
 est_time=(double)Samp_Instant;
 Tim_Var=Tim_Var+(correct_time-est_time)*(correct_time-est_time);
 return(0);
}
/******************************************************************************/
/*                           Compute the carrier phase.
*******************************************************************************/
int Compute_Carrier_Phase()
{
 Est_Phase=atan(Im_Tim_Max/Re_Tim_Max);
 if(Re_Tim_Max < FRAC_ZERO)
  Est_Phase=Est_Phase+M_PI;
 else if((Re_Tim_Max > FRAC_ZERO) && (Im_Tim_Max < FRAC_ZERO))
  Est_Phase=Est_Phase+2.0*M_PI;
 if(!RAND_PHASE)
  Phase_Var=Phase_Var+(Est_Phase-Initial_Phase)*(Est_Phase-Initial_Phase);
 return(0);
}
/******************************************************************************/
/*                           Compute the amplitudes.
*******************************************************************************/
int Compute_Amplitude()
{
 double cost,sint;
 cost=cos(Est_Phase);
 sint=sin(Est_Phase);
 Re_Amp=(Re_Tim_Max*cost+Im_Tim_Max*sint)/(double)PRE_AMB_SIZE;
 Im_Amp=(Im_Tim_Max*cost-Re_Tim_Max*sint)/(double)PRE_AMB_SIZE;
 return(0);
}
/******************************************************************************/
/*                           Compute the variance.
*******************************************************************************/
int Compute_Variance()
{
 int cnt2,cnt3,bit0,bit1;
 double re_mean,im_mean,re_sym,im_sym,re_err,im_err;
 re_mean=Re_Amp*cos(Est_Phase);
 im_mean=Re_Amp*sin(Est_Phase);
 cnt2=Samp_Instant;
 cnt3=ZERO;
 Est_Var=FRAC_ZERO;
 while((cnt2 < RX_SAMP_SIZE) && (cnt3 < PRE_AMB_SIZE))
 {
  bit0=Sym[cnt3]&ONE;
  bit1=Sym[cnt3]>>ONE;
  re_sym=Map[bit0];
  im_sym=Map[bit1];
  re_err=Re_MF_Op[cnt2]*re_sym+Im_MF_Op[cnt2]*im_sym-re_mean;
  im_err=Im_MF_Op[cnt2]*re_sym-Re_MF_Op[cnt2]*im_sym-im_mean;
  Est_Var=Est_Var+re_err*re_err+im_err*im_err;
  cnt2=cnt2+RX_SAMP_PER_SYM;
  cnt3++;
 }
 Est_Var=0.25*Est_Var/(double)PRE_AMB_SIZE;
 Inv_Twice_Noise_Var=1.0/(2.0*Est_Var);
 return(0);
}
/******************************************************************************/
/*                               Print results.
*******************************************************************************/
int Print_Sync_Results()
{
 double mag;
 mag=Re_Mean_Corr*Re_Mean_Corr+Im_Mean_Corr*Im_Mean_Corr;
 mag=mag/(double)(Mean_Corr_Cnt*Mean_Corr_Cnt);
 Re_Amp=0.5*Re_Amp;
 printf("\t Mag. of peak:%lf ",Tim_Max_Mag);
 printf("\t Mag. of avg:%lf\n",mag);
 printf("\t Sampling instant:%d.\n",Samp_Instant);
 printf("\t Act. carrier phase:%lf Est.:%lf.\n",Initial_Phase,Est_Phase);
 printf("\t Re. amp.:%lf Im. amp.:%lf.\n",Re_Amp,Im_Amp);
 printf("\t Est. variance:%lf.\n",Est_Var);
 printf("\t Inv. twice variance:%lf.\n\n",Inv_Twice_Noise_Var);
 return(0);
}
/******************************************************************************/
