/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal1type.h"
#include "deal1ext.h"
/******************************************************************************/
/*                         Convert decimal to binary.
*******************************************************************************/
int Decimal_to_Binary(num,array,array_size)
int num,array[],array_size;
{
 int cnt;
 for(cnt=ZERO;cnt<array_size;cnt++)
  array[cnt]=ZERO;
 cnt=ZERO;
 while(num>ZERO)
 {
  if(cnt == array_size)
   Error_Mes(3);
  array[cnt]=num&1;
  num=num>>1;
  cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                    Convert binary array to decimal.
*******************************************************************************/
int Binary_to_Decimal(array,size)
int array[],size;
{
 int cnt,pow,sum;
 sum=ZERO;
 pow=1;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  sum=sum+pow*array[cnt];
  pow=pow<<1;
 }
 return(sum);
}
/******************************************************************************/
/* Real convolution.
   Input arrays: 'ip1[ip1size]' is first array.
                 'ip2[ip2size]' is second array.
   Output array: 'op[]'.
*******************************************************************************/
int Convolve(ip1,ip2,op,ip1size,ip2size)
int ip1size,ip2size;
double ip1[],ip2[],op[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  op[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    op[opcnt]=op[opcnt]+ip1[ipcnt]*ip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*                      Get the root-raised cosine filter.
*******************************************************************************/
int Get_RRC(size,delay,samp_freq,array)
int size;
double delay,samp_freq,array[];
{
 int inx;
 double time;
 for(inx=ZERO;inx<size;inx++)
 {
  time=(double)inx-delay;
  if(fabs(time) > ERROR)
   Get_Term1(inx,time,samp_freq,array);
  else
   Get_Term2(inx,array);
 }
 return(0);
}
/******************************************************************************/
/*            Get the value which does not correspond to the peak.
*******************************************************************************/
int Get_Term1(inx,time,samp_freq,array)
int inx;
double time,samp_freq,array[];
{
 double theta1,theta2,numer,denom,term1,term2,term3,term4;
 time=time/samp_freq; 
 theta1=MY_PI*(1.0-ROLL_OFF)*time*SYMBOL_RATE;
 theta2=MY_PI*(1.0+ROLL_OFF)*time*SYMBOL_RATE;
 term1=sin(theta1);
 term2=4.0*ROLL_OFF*time*SYMBOL_RATE*cos(theta2);
 numer=term1+term2;
 term3=4.0*ROLL_OFF*time*SYMBOL_RATE;
 term4=term3*term3;
 denom=MY_PI*time*sqrt(SYMBOL_RATE)*(1.0-term4);
 if(fabs(denom) < ERROR)                                    /* 0/0 situation. */
 {
  time=time*samp_freq+DITHER;
  Get_Term1(inx,time,samp_freq,array);                     /* Recursive call. */
 }
 else
  array[inx]=numer/denom;
 return(0);
}
/******************************************************************************/
/*               Get the value which corresponds to the peak.
*******************************************************************************/
int Get_Term2(inx,array)
int inx;
double array[];
{
 array[inx]=(1.0-ROLL_OFF)+4.0*ROLL_OFF/MY_PI;
 return(0);
}
/******************************************************************************/
/*                     Shift the complex array away from zero.
*******************************************************************************/
int Cmplx_Shift_from_Zero(re_array,im_array,size)
int size;
double re_array[],im_array[];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
 {
  re_array[cnt]=re_array[cnt-ONE];
  im_array[cnt]=im_array[cnt-ONE];
 }
 return(0);
}
/******************************************************************************/
/*  Convert uniform random variables to gaussian random variables. Refer to
 *  DSP by Proakis and Manolakis.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r < FRAC_ZERO)
  Error_Mes(5);
 r=sqrt(r);
 theta=2.0*MY_PI*tnarry[UTHETA];
 tnarry[GRAND_0]=r*cos(theta);
 tnarry[GRAND_1]=r*sin(theta);
 return(0);
}
/******************************************************************************/
/*                             Error message.
*******************************************************************************/
int Error_Mes(num)
int num;
{
 fprintf(stdout,"\t Simulation error. Exiting with code %d.\n\n",num);
 exit(num);
 return(0);
}
/******************************************************************************/
