/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal10type.h"
#include "deal10ext.h"
/******************************************************************************/
/*                        Estimate the frequency offset.
*******************************************************************************/
int Get_F_Off()
{
 Scan_Pre_Amb();
 if((Max_F_Off_Inx > MIN_TIM_1) && (Max_F_Off_Inx < MAX_TIM_1))
 {
  Estimate_F_Off();
  if((RAND_TIM == ZERO) && (fabs(CLOCK_ERR) < ERROR))
   Update_Tim_Var();
 }
 Print_F_Off();
 return(0);
}
/******************************************************************************/
/*                          Scan for preamble.
*******************************************************************************/
int Scan_Pre_Amb()
{
 int cnt1,cnt2,inx,max;
 double mag;
 Init_Scan();
 for(cnt1=MIN_TIM;cnt1<MAX_TIM;cnt1++)
 {
  inx=cnt1;
  Re_F_Off_Sum=Im_F_Off_Sum=FRAC_ZERO;
  for(cnt2=ZERO;cnt2<PRE_AMB_SS-ONE;cnt2++)
  {
   if((inx+F_EST_LAG*RX_SAMP_PER_SYM) < RX_SAMP_SIZE)
    Update_F_Off_Sum(inx,cnt2);
   else
    Error_Mes(3);
   inx=inx+RX_SAMP_PER_SYM;
  }
  Process_Sum_Corr(cnt1);
 }
 return(0);
}
/******************************************************************************/
/*                   Initialize scanning of the preamble.
*******************************************************************************/
int Init_Scan()
{
 Max_F_Off_Mag=MIN_MAG;
 Mean_Corr=0.0;
 Mean_Corr_Cnt=ZERO;
 return(0);
}
/******************************************************************************/
/*                      Update the differential sum.
*******************************************************************************/
int Update_F_Off_Sum(inx1,cnt2)
int inx1,cnt2;
{
 int inx2,im_bit1[ISI_LEN],im_bit2[ISI_LEN];
 double re_samp1,re_samp2,im_samp1,im_samp2,re_diff,im_diff,re_sym1,re_sym2;
 double im_sym1,im_sym2;
 inx2=inx1+F_EST_LAG*RX_SAMP_PER_SYM;
 re_sym1=Map[Sym[cnt2]&ONE];
 re_sym2=Map[Sym[cnt2+F_EST_LAG]&ONE];
 im_sym1=Im_ISI[cnt2];
 im_sym2=Im_ISI[cnt2+F_EST_LAG];
 re_samp1=re_sym1*Re_MF_Op[inx1]+im_sym1*Im_MF_Op[inx1];
 im_samp1=re_sym1*Im_MF_Op[inx1]-im_sym1*Re_MF_Op[inx1];
 re_samp2=re_sym2*Re_MF_Op[inx2]+im_sym2*Im_MF_Op[inx2];
 im_samp2=re_sym2*Im_MF_Op[inx2]-im_sym2*Re_MF_Op[inx2];
 re_diff=re_samp2*re_samp1+im_samp2*im_samp1;
 im_diff=im_samp2*re_samp1-re_samp2*im_samp1;
 Re_F_Off_Sum=Re_F_Off_Sum+re_diff;
 Im_F_Off_Sum=Im_F_Off_Sum+im_diff;
 return(0);
}
/******************************************************************************/
/*                     Process the output of correlator 1.
*******************************************************************************/
int Process_Sum_Corr(cnt1)
int cnt1;
{
 double mag;
 mag=Re_F_Off_Sum*Re_F_Off_Sum+Im_F_Off_Sum*Im_F_Off_Sum;
 if(Frame_Cnt == PRINT_FRAME)
  fprintf(fp2,"%lf\n",mag);
 if(mag > Max_F_Off_Mag)
  Store_Max(cnt1,mag);
 Mean_Corr=Mean_Corr+mag;
 Mean_Corr_Cnt++;
 return(0);
}
/******************************************************************************/
/*              Store the maximum value of the differential sum.
*******************************************************************************/
int Store_Max(inx,mag)
int inx;
double mag;
{
 Max_F_Off_Mag=mag;
 Max_F_Off_Inx=inx;
 Max_Re_F_Off_Sum=Re_F_Off_Sum;
 Max_Im_F_Off_Sum=Im_F_Off_Sum;
 return(0);
}
/******************************************************************************/
/*                      Estimate the frequency offset.
*******************************************************************************/
int Estimate_F_Off()
{
 double theta,diff;
 if(fabs(Max_Im_F_Off_Sum) < fabs(Max_Re_F_Off_Sum))
 {
  theta=atan(Max_Im_F_Off_Sum/Max_Re_F_Off_Sum);
  if((Max_Im_F_Off_Sum > FRAC_ZERO) && (Max_Re_F_Off_Sum < FRAC_ZERO))
   theta=theta+M_PI;
  else if((Max_Im_F_Off_Sum < FRAC_ZERO) && (Max_Re_F_Off_Sum < FRAC_ZERO))
   theta=theta-M_PI;
 }
 else
 {
  theta=M_PI/2.0-atan(Max_Re_F_Off_Sum/Max_Im_F_Off_Sum);
  if(Max_Im_F_Off_Sum < FRAC_ZERO)
   theta=theta-M_PI;
 }
 Est_F_Off=theta/(double)(TX_IP_FAC*F_EST_LAG);
 diff=fabs(Est_F_Off-F_OFF);
 F_Off_RMS=F_Off_RMS+diff*diff;
 if(diff > Max_F_Off_Err)
  Max_F_Off_Err=diff;
 return(0);
}
/******************************************************************************/
/*                        Update the timing variance.
*******************************************************************************/
int Update_Tim_Var()
{
 int tim_err;
 double correct_time,est_time;
 correct_time=(double)IDEAL_TIM_INSTANT;
 est_time=(double)Max_F_Off_Inx;
 Tim_Var=Tim_Var+(correct_time-est_time)*(correct_time-est_time);
 tim_err=abs(Max_F_Off_Inx-IDEAL_TIM_INSTANT);
 if(tim_err > Max_Tim_Err)
  Max_Tim_Err=tim_err;
 return(0);
}
/******************************************************************************/
/*                        Print the frequency offset.
*******************************************************************************/
int Print_F_Off()
{
 double mag,ratio;
 mag=Mean_Corr;
 if(Mean_Corr_Cnt != ZERO)
  mag=mag/(double)(Mean_Corr_Cnt);
 else
  Error_Mes(1);
 ratio=Max_F_Off_Mag/mag;
 Avg_Ratio=Avg_Ratio+ratio;
 if(ratio < Min_Ratio)
  Min_Ratio=ratio;
 printf("\t Frame:%d\n",Frame_Cnt);
 printf("\t 1st attempt.\n");
 printf("\t Time:%d Max Re:%lf ",Max_F_Off_Inx,Max_Re_F_Off_Sum);
 printf("Max Im:%lf ",Max_Im_F_Off_Sum);
 printf("Freq Off:%lf\n",Est_F_Off);
 printf("\t Mag. of peak:%lf ",Max_F_Off_Mag);
 printf(" Mag. of avg:%lf.\n",mag);
 printf("\t Samples:%d Ratio:%lf\n",Mean_Corr_Cnt,ratio);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
