/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal10type.h"
#include "deal10ext.h"
/******************************************************************************/
/*               Estimate the frequency offset - 2nd attempt.
*******************************************************************************/
int Get_F_Off_2()
{
 Scan_Pre_Amb_2();
 if((Max_F_Off_Inx_2 > MIN_TIM_1) && (Max_F_Off_Inx_2 < MAX_TIM_1))
 {
  Estimate_F_Off_2();
  if((RAND_TIM == ZERO) && (fabs(CLOCK_ERR) < ERROR))
   Update_Tim_Var_2();
 }
 Print_F_Off_2();
 return(0);
}
/******************************************************************************/
/*                          Scan for preamble.
*******************************************************************************/
int Scan_Pre_Amb_2()
{
 int cnt1,cnt2,inx,max;
 double mag;
 Init_Scan_2();
 for(cnt1=MIN_TIM;cnt1<MAX_TIM;cnt1++)
 {
  inx=cnt1;
  Re_F_Off_Sum=Im_F_Off_Sum=FRAC_ZERO;
  for(cnt2=ZERO;cnt2<PRE_AMB_SS-ONE;cnt2++)
  {
   if((inx+F_EST_LAG*RX_SAMP_PER_SYM) < RX_SAMP_SIZE)
    Update_F_Off_Sum(inx,cnt2);
   else
    Error_Mes(3);
   inx=inx+RX_SAMP_PER_SYM;
  }
  Process_Sum_Corr_2(cnt1);
 }
 return(0);
}
/******************************************************************************/
/*                   Initialize scanning of the preamble.
*******************************************************************************/
int Init_Scan_2()
{
 Max_F_Off_Mag=MIN_MAG;
 Mean_Corr=0.0;
 Mean_Corr_Cnt=ZERO;
 return(0);
}
/******************************************************************************/
/*                     Process the output of correlator 2.
*******************************************************************************/
int Process_Sum_Corr_2(cnt1)
int cnt1;
{
 double mag;
 mag=Re_F_Off_Sum*Re_F_Off_Sum+Im_F_Off_Sum*Im_F_Off_Sum;
 if(Frame_Cnt == PRINT_FRAME)
  fprintf(fp3,"%lf\n",mag);
 if(mag > Max_F_Off_Mag)
  Store_Max_2(cnt1,mag);
 Mean_Corr=Mean_Corr+mag;
 Mean_Corr_Cnt++;
 return(0);
}
/******************************************************************************/
/*              Store the maximum value of the differential sum.
*******************************************************************************/
int Store_Max_2(inx,mag)
int inx;
double mag;
{
 Max_F_Off_Mag=mag;
 Max_F_Off_Inx_2=inx;
 Max_Re_F_Off_Sum=Re_F_Off_Sum;
 Max_Im_F_Off_Sum=Im_F_Off_Sum;
 return(0);
}
/******************************************************************************/
/*                      Estimate the frequency offset.
*******************************************************************************/
int Estimate_F_Off_2()
{
 double theta,diff;
 if(fabs(Max_Im_F_Off_Sum) < fabs(Max_Re_F_Off_Sum))
 {
  theta=atan(Max_Im_F_Off_Sum/Max_Re_F_Off_Sum);
  if((Max_Im_F_Off_Sum > FRAC_ZERO) && (Max_Re_F_Off_Sum < FRAC_ZERO))
   theta=theta+M_PI;
  else if((Max_Im_F_Off_Sum < FRAC_ZERO) && (Max_Re_F_Off_Sum < FRAC_ZERO))
   theta=theta-M_PI;
 }
 else
 {
  theta=M_PI/2.0-atan(Max_Re_F_Off_Sum/Max_Im_F_Off_Sum);
  if(Max_Im_F_Off_Sum < FRAC_ZERO)
   theta=theta-M_PI;
 }
 Est_F_Off_2=theta/(double)(TX_IP_FAC*F_EST_LAG);
 if(IDEAL_CAR == ZERO)
  diff=fabs(F_OFF-Est_F_Off-Est_F_Off_2);
 else
  diff=fabs(Est_F_Off_2);
 F_Off_RMS_2=F_Off_RMS_2+diff*diff;
 if(diff > Max_F_Off_Err_2)
  Max_F_Off_Err_2=diff;
 return(0);
}
/******************************************************************************/
/*                        Update the timing variance.
*******************************************************************************/
int Update_Tim_Var_2()
{
 int tim_err;
 double correct_time,est_time;
 correct_time=(double)IDEAL_TIM_INSTANT;
 est_time=(double)Max_F_Off_Inx_2;
 Tim_Var_2=Tim_Var_2+(correct_time-est_time)*(correct_time-est_time);
 tim_err=abs(Max_F_Off_Inx_2-IDEAL_TIM_INSTANT);
 if(tim_err > Max_Tim_Err_2)
  Max_Tim_Err_2=tim_err;
 return(0);
}
/******************************************************************************/
/*                        Print the frequency offset.
*******************************************************************************/
int Print_F_Off_2()
{
 double mag,ratio;
 mag=Mean_Corr;
 if(Mean_Corr_Cnt != ZERO)
  mag=mag/(double)(Mean_Corr_Cnt);
 else
  Error_Mes(1);
 ratio=Max_F_Off_Mag/mag;
 Avg_Ratio_2=Avg_Ratio_2+ratio;
 if(ratio < Min_Ratio_2)
  Min_Ratio_2=ratio;
 printf("\t 2nd attempt.\n");
 printf("\t Time:%d Max Re:%lf ",Max_F_Off_Inx_2,Max_Re_F_Off_Sum);
 printf("Max Im:%lf ",Max_Im_F_Off_Sum);
 printf("Freq Off:%lf\n",Est_F_Off_2);
 printf("\t Mag. of peak:%lf ",Max_F_Off_Mag);
 printf(" Mag. of avg:%lf.\n",mag);
 printf("\t Samples:%d Ratio:%lf\n",Mean_Corr_Cnt,Max_F_Off_Mag/mag);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
