/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
FILE *fp1,*fp2,*fp3,*fp4,*fp5,*fp6,*fp7;                    /* File pointers. */
/******************************************************************************/
/*                     VARIABLES RELATED TO THE FRAME.
*******************************************************************************/
int Ber;                                   /* Bit-error-rate of uncoded QPSK. */
int T_Ber;                             /* Bit-error-rate of turbo coded QPSK. */
int Valid_Bit;                                          /* Valid bit counter. */
int Erasure;                                                /* Frame erasure. */
int Tx_Samp_Cnt;                                       /* Sample count at tx. */
int Rx_Samp_Cnt;                                       /* Sample count at rx. */
int Frame_Cnt;                                                /* Frame count. */
int Bits[DATA_SIZE];                                     /* Information bits. */
int Sym[FRAME_SIZE];                                       /* Stores symbols. */
double Map[BPSK_SIZE];                                /* Stores BPSK mapping. */
/******************************************************************************/
/*                    VARIABLES RELATED TO THE TRELLIS.
*******************************************************************************/
int Enc_State;                                              /* Encoder state. */
int Enc_N[ENC_SIZE];                           /* Stores encoder numer. taps. */
int Enc_D[ENC_SIZE];                           /* Stores encoder denom. taps. */
int Trel_NS[NUM_STATES][NUM_INPUTS];                           /* Next state. */
int Trel_PS[NUM_STATES][NUM_INPUTS];                           /* Prev state. */
int Trel_Op[NUM_STATES][NUM_INPUTS];                              /* Outputs. */
/******************************************************************************/
/*                    VARIABLES RELATED TO INTERLEAVER.
*******************************************************************************/
int Iv_Map[IV_SIZE];                                  /* Interleaver mapping. */
int De_Iv_Map[IV_SIZE];                             /* Deinterleaver mapping. */
unsigned short Iv_Noise_Store[PN_SIZE];                 /* Stores PNS status. */
/******************************************************************************/
/*                        TRANSMIT FILTER VARIABLES.
*******************************************************************************/
double Rand_Tx_Delay;                                        /* Random delay. */
double Re_Tx_Op;                                              /* Re. tx. o/p. */
double Im_Tx_Op;                                              /* Im. tx. o/p. */
double Tx_Flt[TX_FLT_SIZE];                               /* Transmit filter. */
double Re_Tx_Reg[TX_FLT_SIZE];                                /* Re. tx. reg. */
double Im_Tx_Reg[TX_FLT_SIZE_1];                              /* Im. tx. reg. */
/******************************************************************************/
/*              VARIABLES RELATED TO CLOCK ERROR AT TRANSMITTER.
*******************************************************************************/
int No_Right_Shift_Flag;         /* 1: Dont right shift tx. reg.;0: otherwise */
int Compute_Tx_Op_Flag;                   /* 1: Compute tx. o/p;0: otherwise. */
int Extra_Iter;                           /* 1: Extra iteration;0: otherwise. */
double Tx_Samp_Err;                                /* Time err. in tx. coeff. */
/******************************************************************************/
/*                  VARIABLES RELATED TO THE MODULATED SIGNAL.
*******************************************************************************/
double Passband_Signal[TX_SAMP_SIZE];                     /* Passband signal. */
double Initial_Phase;                         /* Initial carrier phase at tx. */
double Theta;                                    /* Starting phase at the tx. */
/******************************************************************************/
/*                       NOISE GENERATOR VARIABLES.
*******************************************************************************/
unsigned short N1_Store[PN_SIZE];                       /* Stores PNS status. */
unsigned short N2_Store[PN_SIZE];                       /* Stores PNS status. */
double Variance;                                           /* Noise variance. */
double Inv_Twice_Noise_Var;           /* Inverse of twice the noise variance. */
double N1_Val[NVALSIZE];                         /* Contains U & G rand. var. */
/******************************************************************************/
/*              VARIABLES RELATED TO FREQUENCY OFFSET ESTIMATION.
*******************************************************************************/
int Max_F_Off_Inx;                            /* Index corresponding to peak. */
int Max_Tim_Err;                          /* Offset from ideal samp. instant. */
double ISI_Coeff[ISI_LEN];                                      /* ISI coeff. */
double ISI_Reg[ISI_LEN];                             /* Contains ISI symbols. */
double Im_ISI[PRE_AMB_SIZE];                                /* Im. ISI value. */
double Re_F_Off_Sum;                                        /* Re. diff. sum. */
double Im_F_Off_Sum;                                        /* Im. diff. sum. */
double Max_F_Off_Mag;                              /* Max. mag. of diff. sum. */
double Max_Re_F_Off_Sum;                               /* Max. re. diff. sum. */
double Max_Im_F_Off_Sum;                               /* Max. im. diff. sum. */
double Est_F_Off;                                  /* Estimated freq. offset. */
double F_Off_RMS;                                 /* RMS value of est. error. */
double Max_F_Off_Err;                          /* Absolute max. error in est. */
double Min_Ratio;                           /* Min. ratio of peak to average. */
double Avg_Ratio;                           /* Avg. ratio of peak to average. */
/******************************************************************************/
/*       VARIABLES RELATED TO FREQUENCY OFFSET ESTIMATION: 2nd ATTEMPT.
*******************************************************************************/
int Max_F_Off_Inx_2;             /* 2nd attempt: Index corresponding to peak. */
int Max_Tim_Err_2;                    /* 2nd offset from ideal samp. instant. */
double Est_F_Off_2;                   /* 2nd attempt: Estimated freq. offset. */
double F_Off_RMS_2;                  /* 2nd attempt: RMS value of est. error. */
double Max_F_Off_Err_2;           /* 2nd attempt: Absolute max. error in est. */
double Min_Ratio_2;            /* 2nd attempt: Min. ratio of peak to average. */
double Avg_Ratio_2;            /* 2nd attempt: Avg. ratio of peak to average. */
/******************************************************************************/
/*     VARIABLES RELATED TO FREQUENCY OFFSET ESTIMATION: 3rd ATTEMPT (ML).
*******************************************************************************/
double Est_F_Off_3;                   /* 3rd attempt: Estimated freq. offset. */
double F_Off_RMS_3;                  /* 3rd attempt: RMS value of est. error. */
double Max_F_Off_Err_3;           /* 3rd attempt: Absolute max. error in est. */
/******************************************************************************/
/*     VARIABLES RELATED TO FREQUENCY OFFSET ESTIMATION: 4th ATTEMPT (ML).
*******************************************************************************/
double F_Off_RMS_4;                  /* 4th attempt: RMS value of est. error. */
double Max_F_Off_Err_4;           /* 4th attempt: Absolute max. error in est. */
/******************************************************************************/
/*           VARIABLES RELATED TO ML FREQUENCY OFFSET ESTIMATION.
*******************************************************************************/
double Est_F_Off_ML;                          /* ML estimate of freq. offset. */
double Re_DFT;                                                    /* Re. DFT. */
double Im_DFT;                                                    /* Im. DFT. */
double Max_DFT_Mag;                                           /* Peak of DFT. */
double ML_Step;                                                /* Freq. step. */
/******************************************************************************/
/*                TIMING RECOVERY VARIABLES (CORRELATION METHOD).
*******************************************************************************/
int Samp_Instant;                                        /* Sampling instant. */
int Max_Tim_Err_3;                    /* 3rd offset from ideal samp. instant. */
int Mean_Corr_Cnt;                       /* Ctr. for the no. of correlations. */
double Re_Tim_Corr;                                /* Re. timing correlation. */
double Im_Tim_Corr;                                /* Im. timing correlation. */
double Mean_Corr;                         /* Mean of the correlation sq. mag. */
double Re_Tim_Max;                                   /* Re. correlation peak. */
double Im_Tim_Max;                                   /* Im. correlation peak. */
double Tim_Max_Mag;                         /* Max. magnitude of correlation. */
double Min_Ratio_3;            /* 3rd attempt: Min. ratio of peak to average. */
double Avg_Ratio_3;            /* 3rd attempt: Avg. ratio of peak to average. */
/******************************************************************************/
/*               LOCAL OSCILLATOR (AT THE RECEIVER) VARIABLES.
*******************************************************************************/
double C_Theta;                                             /* Carrier phase. */
double Re_LO[LO_PERIOD];                                /* Cosine term of LO. */
double Im_LO[LO_PERIOD];                                  /* Sine term of LO. */
double Re_LO_Op;                                               /* Re. LO o/p. */
double Im_LO_Op;                                               /* Im. LO o/p. */
/******************************************************************************/
/*                         MATCHED FILTER VARIABLES.
*******************************************************************************/
double Rx_Re_Sym1;                                         /* Rx. re. symbol. */
double Rx_Im_Sym1;                                         /* Rx. im. symbol. */
double Rx_Re_Sym2;                                  /* Rx. re. offset symbol. */
double Rx_Im_Sym2;                                  /* Rx. im. offset symbol. */
double Rx_Re_Sym3;                                         /* Rx. re. symbol. */
double Rx_Im_Sym3;                                         /* Rx. im. symbol. */
double MF[MF_SIZE];                                        /* Matched filter. */
double Re_MF_Reg[MF_SIZE];                                     /* Re. MF reg. */
double Im_MF_Reg[MF_SIZE];                                     /* Im. MF reg. */
double Re_MF_Op[RX_SAMP_SIZE];                                 /* Re. MF o/p. */
double Im_MF_Op[RX_SAMP_SIZE];                                 /* Im. MF o/p. */
/******************************************************************************/
/*     TIMING, CARRIER, AMPLITUDE AND NOISE VARIANCE ESTIMATION VARIABLES.
*******************************************************************************/
int Sym_Cnt;                                                 /* Symbol count. */
double Est_Phase;                                 /* Estimated carrier phase. */
double Cost;                                                 /* Cosine value. */
double Sint;                                                   /* Sine value. */
double Tim_Var;                                          /* Var. of tim. est. */
double Tim_Var_2;                           /* 2nd attempt: Var. of tim. est. */
double Tim_Var_3;                           /* 3rd attempt: Var. of tim. est. */
double Re_Amp;                                              /* Re. amplitude. */
double Im_Amp;                                              /* Im. amplitude. */
double Avg_Amp1;                                           /* Avg. amplitude. */
double Est_Var;                                        /* Estimated variance. */
double Amp_RMS_Err_1;                /* RMS error in amp. est. from tim. rec. */
double NVar_RMS_Err_1;          /* RMS error in noise var. est from tim. rec. */
/******************************************************************************/
/*              VARIABLES RELATED TO CLOCK ERROR AT RECEIVER.
*******************************************************************************/
int Tim_Track_Instant;                        /* Tracks the sampling instant. */
int Tim_Offset;                  /* Offset from the current sampling instant. */
int Cum_Tim_Offset;                             /* Cummulative timing offset. */
int Tim_Track;                               /* 1: Track timing;0: otherwise. */
int Nxt_Tim_Track_Instant;         /* Next instant to start tracking the tim. */
int Tim_Reg[TIM_REG_SIZE];                 /* Tim. reg. to store the offsets. */
double Avg_Re_Amp[TIM_WIN_LEN];                /* Avg. re. amp. to track tim. */
double C_Avg_Re_Amp;                /* Avg. re. amp. for car. phase tracking. */
double C_Avg_Im_Amp;                /* Avg. im. amp. for car. phase tracking. */
double Re_Sym_Buf[SYM_BUF_LEN_P1];             /* Stores re. part of symbols. */
double Re_MF_Op_Buf[TIM_WIN_LEN][SYM_BUF_LEN_P1];        /* Store re. MF o/p. */
double Im_MF_Op_Buf[TIM_WIN_LEN][SYM_BUF_LEN_P1];        /* Store im. MF o/p. */
/******************************************************************************/
/*                            RECEIVED SYMBOLS.
*******************************************************************************/
double Sym0[DATA_SIZE];                                       /* Data symbol. */
double Sym1[DATA_SIZE];                                     /* Parity symbol. */
double Sym2[DATA_SIZE];                                     /* Parity symbol. */
/******************************************************************************/
/*                            BCJR VARIABLES.
*******************************************************************************/
int Iter;                                               /* Counts iterations. */
double Alpha[DATA_SIZE][NUM_STATES];                               /* Alphas. */
double Beta[DATA_SIZE_P1][NUM_STATES];                              /* Betas. */
double Gamma_I[DATA_SIZE][NUM_INPUTS];                 /* Dv. intrinsic info. */
double Gamma_E_P1[DATA_SIZE][NUM_INPUTS];              /* Extrinsic info: P1. */
double Gamma_E_P2[DATA_SIZE][NUM_INPUTS];              /* Extrinsic info: P2. */
double Prob[DATA_SIZE][NUM_INPUTS];                    /* Data probabilities. */
/******************************************************************************/
