/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal10type.h"
#include "deal10ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Init_Frame();
  Get_Frame();                                           /* Go to deal10sym.c */
  Get_Tx_Flt_Op();
  Receiver_1();                                          /* Demod. with PI/2. */
  Get_F_Off();                        /* 1st est. of timing and freq. offset. */
  Receiver_2();                  /* Demod with PI/2+1st est. of freq. offset. */
  Get_F_Off_2();                      /* 2nd est. of timing and freq. offset. */
  Get_ML_F_Off();                                     /* ML freq. offset est. */
  Receiver_2_1();                /* Final est. of timing and BCJR parameters. */
  Check_for_Frame_Erasure();
  if(SNR_B < TC_SNR_THRESH)
   Turbo_Dec();
  printf("\t Cummulative timing offset:%d\n\n",Cum_Tim_Offset);
 }
 return(0);
}
/******************************************************************************/
/*                    Initialize the variables in a frame.
*******************************************************************************/
int Init_Frame()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  Re_MF_Reg[cnt]=Im_MF_Reg[cnt]=FRAC_ZERO;
 if(RAND_TIM == ONE)
  Rand_Tx_Delay=(double)TX_DELAY+(double)TX_IP_FAC*erand48(N1_Store);
 else
  Rand_Tx_Delay=(double)TX_DELAY;
 if(RAND_PHASE == ONE)
  Initial_Phase=Theta=2.0*M_PI*erand48(N1_Store);
 else
  Initial_Phase=Theta=0.0*M_PI;
 Tx_Samp_Cnt=Rx_Samp_Cnt=ZERO;
 Extra_Iter=ZERO;
 No_Right_Shift_Flag=FALSE;
 Compute_Tx_Op_Flag=TRUE;
 Tx_Samp_Err=FRAC_ZERO;
 Tx_Reg_Init();
 return(0);
}
/******************************************************************************/
/*                       Get the transmit filter output.
*******************************************************************************/
int Get_Tx_Flt_Op()
{
 int cnt1,bit0,bit1,ip_cnt;
 double re_sym,im_sym;
 for(cnt1=ZERO;cnt1<FRAME_SIZE;cnt1++)
 {
  bit0=Sym[cnt1]&ONE;                                                 /* LSB. */
  bit1=Sym[cnt1]>>ONE;                                                /* MSB. */
  re_sym=Map[bit0];                                 /* LSB maps to real-part. */
  im_sym=Map[bit1];                            /* MSB maps to imaginary part. */
  for(ip_cnt=ZERO;ip_cnt<TX_IP_FAC+Extra_Iter;ip_cnt++)
  {
   Get_Tx_Flt();
   Compute_Tx_Op(ip_cnt,re_sym,im_sym);
   if(No_Right_Shift_Flag == FALSE)
    re_sym=im_sym=0.0;
  }
  Extra_Iter=ZERO;
 }
 return(0);
}
/******************************************************************************/
/*                        Get the transmit filter.
*******************************************************************************/
int Get_Tx_Flt()
{
 if(No_Right_Shift_Flag == TRUE)
 {
  No_Right_Shift_Flag=FALSE;
  Compute_Tx_Op_Flag=TRUE;
  Get_RRC_Clk_Err(TX_FLT_SIZE,Rand_Tx_Delay,TX_SAMP_FREQ,Tx_Flt,Tx_Samp_Err);
  Normalize_RRC(Tx_Flt,TX_FLT_SIZE);
 }
 else if(Compute_Tx_Op_Flag == FALSE)
 {
  No_Right_Shift_Flag=FALSE;
  Compute_Tx_Op_Flag=TRUE;
 }
 else if(Tx_Samp_Err < -TX_SAMP_PERIOD)
 {
  Tx_Samp_Err=Tx_Samp_Err+TX_SAMP_PERIOD;
  No_Right_Shift_Flag=TRUE;
  Compute_Tx_Op_Flag=TRUE;
  Extra_Iter=TRUE;
  Get_RRC_Clk_Err(TX_FLT_SIZE,Rand_Tx_Delay,TX_SAMP_FREQ,Tx_Flt,Tx_Samp_Err);
  Normalize_RRC(Tx_Flt,TX_FLT_SIZE);
 }
 else if(Tx_Samp_Err > TX_SAMP_PERIOD)
 {
  Tx_Samp_Err=Tx_Samp_Err-TX_SAMP_PERIOD;
  No_Right_Shift_Flag=FALSE;
  Compute_Tx_Op_Flag=FALSE;
  Get_RRC_Clk_Err(TX_FLT_SIZE,Rand_Tx_Delay,TX_SAMP_FREQ,Tx_Flt,Tx_Samp_Err);
  Normalize_RRC(Tx_Flt,TX_FLT_SIZE);
 }
 else
 {
  No_Right_Shift_Flag=FALSE;
  Compute_Tx_Op_Flag=TRUE;
  Get_RRC_Clk_Err(TX_FLT_SIZE,Rand_Tx_Delay,TX_SAMP_FREQ,Tx_Flt,Tx_Samp_Err);
  Normalize_RRC(Tx_Flt,TX_FLT_SIZE);
 }
 return(0);
}
/******************************************************************************/
/*             Compute the current output of the transmit filter.
*******************************************************************************/
int Compute_Tx_Op(ip_cnt,re_sym,im_sym)
int ip_cnt;
double re_sym,im_sym;
{
 int cnt;
 if(No_Right_Shift_Flag == FALSE)
 {
  Cmplx_Shift_from_Zero_Tx(Re_Tx_Reg,Im_Tx_Reg,TX_FLT_SIZE,TX_OFF);
  Re_Tx_Reg[ZERO]=re_sym;
  Im_Tx_Reg[ZERO]=im_sym;
 }
 if(Compute_Tx_Op_Flag == TRUE)
 {
  Re_Tx_Op=Im_Tx_Op=0.0;
  for(cnt=ip_cnt;cnt<TX_FLT_SIZE;cnt=cnt+TX_IP_FAC)
   Re_Tx_Op=Re_Tx_Op+Tx_Flt[cnt]*Re_Tx_Reg[cnt];
  for(cnt=ip_cnt;cnt<TX_FLT_SIZE_1;cnt=cnt+TX_IP_FAC)
  {
   if((cnt-TX_OFF) >= ZERO)
    Im_Tx_Op=Im_Tx_Op+Tx_Flt[cnt-TX_OFF]*Im_Tx_Reg[cnt];
  }
  Modulate();
 }
 return(0);
}
/******************************************************************************/
/*               Modulate the baseband signal and add noise.
*******************************************************************************/
int Modulate()
{
 double erand48();
 if(Tx_Samp_Cnt == TX_SAMP_SIZE)
  Error_Mes(3);
 if(Frame_Cnt == PRINT_FRAME)
  fprintf(fp5,"%lf\n",sqrt(Re_Tx_Op*Re_Tx_Op+Im_Tx_Op*Im_Tx_Op));
 Passband_Signal[Tx_Samp_Cnt]=Re_Tx_Op*cos(Theta)-Im_Tx_Op*sin(Theta);
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);                             /* Go to deal10util.c */
 Passband_Signal[Tx_Samp_Cnt]=Passband_Signal[Tx_Samp_Cnt]+N1_Val[GRAND_0];
 Theta=Theta+0.5*M_PI+F_OFF;
 if(Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 Tx_Samp_Cnt++;
 Tx_Samp_Err=Tx_Samp_Err+SAMP_ERR;
 return(0);
}
/******************************************************************************/
/*                            First iteration.
*******************************************************************************/
int Receiver_1()
{
 int cnt;
 Init_Rx();
 for(cnt=ZERO;cnt<TWO*PRE_AMB_SIZE*TX_IP_FAC;cnt++)
 {
  Demodulate_F_Off(cnt);
  Get_MF_Op();
 }
 return(0);
}
/******************************************************************************/
/*                          Initialize receiver.
*******************************************************************************/
int Init_Rx()
{
 int cnt;
 C_Theta=FRAC_ZERO;
 Rx_Samp_Cnt=ZERO;
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  Re_MF_Reg[cnt]=Im_MF_Reg[cnt]=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
/*                      Demodulate the passband signal.
*******************************************************************************/
int Demodulate_F_Off(cnt)
int cnt;
{
 Re_LO_Op=Passband_Signal[cnt]*Re_LO[cnt%LO_PERIOD];
 Im_LO_Op=Passband_Signal[cnt]*Im_LO[cnt%LO_PERIOD];
 return(0);
}
/******************************************************************************/
/*                            Get the MF output.
*******************************************************************************/
int Get_MF_Op()
{
 int cnt;
 for(cnt=ZERO;cnt<RX_IP_FAC;cnt++)
 {
  Cmplx_Shift_from_Zero(Re_MF_Reg,Im_MF_Reg,MF_SIZE);   /* Go to deal10util.c */
  Re_MF_Reg[ZERO]=Re_LO_Op;
  Im_MF_Reg[ZERO]=Im_LO_Op;
  Compute_MF_Op(cnt);
  Re_LO_Op=Im_LO_Op=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the current MF output.
*******************************************************************************/
int Compute_MF_Op(ip_cnt)
int ip_cnt;
{
 int cnt;
 if(Rx_Samp_Cnt == RX_SAMP_SIZE)
  Error_Mes(3);
 Re_MF_Op[Rx_Samp_Cnt]=Im_MF_Op[Rx_Samp_Cnt]=0.0;
 for(cnt=ip_cnt;cnt<MF_SIZE;cnt=cnt+RX_IP_FAC)
 {
  Re_MF_Op[Rx_Samp_Cnt]=Re_MF_Op[Rx_Samp_Cnt]+MF[cnt]*Re_MF_Reg[cnt];
  Im_MF_Op[Rx_Samp_Cnt]=Im_MF_Op[Rx_Samp_Cnt]+MF[cnt]*Im_MF_Reg[cnt];
 }
 Rx_Samp_Cnt++;
 return(0);
}
/******************************************************************************/
/*                            Second iteration.
*******************************************************************************/
int Receiver_2()
{
 int cnt;
 Init_Rx();
 for(cnt=ZERO;cnt<TWO*PRE_AMB_SIZE*TX_IP_FAC;cnt++)
 {
  Demodulate_2(cnt);
  Get_MF_Op();
 }
 return(0);
}
/******************************************************************************/
/*                     Second iteration of demodulation.
*******************************************************************************/
int Demodulate_2(cnt)
int cnt;
{
 Re_LO_Op=  2.0*Passband_Signal[cnt]*cos(C_Theta);
 Im_LO_Op= -2.0*Passband_Signal[cnt]*sin(C_Theta);
 if(IDEAL_CAR == ZERO)
  C_Theta=C_Theta+0.5*M_PI+Est_F_Off;
 else
  C_Theta=C_Theta+0.5*M_PI;
 if(C_Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 return(0);
}
/******************************************************************************/
/*                            Iteration 2.1.
*******************************************************************************/
int Receiver_2_1()
{
 int cnt;
 Init_Rx();
 for(cnt=ZERO;cnt<TWO*PRE_AMB_SIZE*TX_IP_FAC;cnt++)
 {
  Demodulate_3(cnt);
  Get_MF_Op();
 }
 Get_Sync();
 return(0);
}
/******************************************************************************/
/*                         Check for frame erasure.
*******************************************************************************/
int Check_for_Frame_Erasure()
{
 int samp_time;
 if(IDEAL_TIM == 1)
  Tim_Track_Instant=IDEAL_TIM_INSTANT;
 else
  Tim_Track_Instant=Samp_Instant;
 if((Tim_Track_Instant > MIN_TIM) && (Tim_Track_Instant < MAX_TIM))
  Receiver_3();
 else                                                       /* Frame erasure. */
  Erasure++;
 return(0);
}
/******************************************************************************/
/*                            Third iteration.
*******************************************************************************/
int Receiver_3()
{
 int cnt;
 Init_Rx_3();
 for(cnt=ZERO;cnt<TX_SAMP_SIZE;cnt++)
 {
  Demodulate_3(cnt);
  Get_MF_Op_3();
 }
 return(0);
}
/******************************************************************************/
/*                          Initialize receiver. 3rd attempt.
*******************************************************************************/
int Init_Rx_3()
{
 int cnt;
 C_Theta=FRAC_ZERO;
 Sym_Cnt=Rx_Samp_Cnt=ZERO;
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  Re_MF_Reg[cnt]=Im_MF_Reg[cnt]=FRAC_ZERO;
 for(cnt=ZERO;cnt<TIM_WIN_LEN;cnt++)
  Avg_Re_Amp[cnt]=FRAC_ZERO;
 C_Avg_Re_Amp=C_Avg_Im_Amp=FRAC_ZERO;
 Cum_Tim_Offset=Tim_Offset=ZERO;
 Tim_Track=TRUE;
 return(0);
}
/******************************************************************************/
/*                     Third iteration of demodulation.
*******************************************************************************/
int Demodulate_3(cnt)
int cnt;
{
 Re_LO_Op=  2.0*Passband_Signal[cnt]*cos(C_Theta);
 Im_LO_Op= -2.0*Passband_Signal[cnt]*sin(C_Theta);
 if(IDEAL_CAR == ZERO)
  C_Theta=C_Theta+0.5*M_PI+Est_F_Off+Est_F_Off_3;
 else
  C_Theta=C_Theta+0.5*M_PI;
 if(C_Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 return(0);
}
/******************************************************************************/
/*  Get the MF output. Since the timing has been acquired, the MF output
    needs to be computed only at the right instants.
*******************************************************************************/
int Get_MF_Op_3()
{
 int ip_cnt,tim_win_inx;
 for(ip_cnt=ZERO;ip_cnt<RX_IP_FAC;ip_cnt++)
 {
  Cmplx_Shift_from_Zero(Re_MF_Reg,Im_MF_Reg,MF_SIZE);    /* Go to deal6util.c */
  Re_MF_Reg[ZERO]=Re_LO_Op;
  Im_MF_Reg[ZERO]=Im_LO_Op;
  if(Rx_Samp_Cnt >= (Tim_Track_Instant-TIM_WINDOW))
  {
   tim_win_inx=Rx_Samp_Cnt-Tim_Track_Instant;
   if(abs(tim_win_inx) <= TIM_WINDOW)
   {
    Compute_MF_Op_3(ip_cnt);
    Sample_MF_Op(tim_win_inx);
   }
   if(tim_win_inx == TIM_WINDOW)
   {
    Tim_Track_Instant=Tim_Track_Instant+RX_OFF+Tim_Offset;
    Tim_Offset=ZERO;
    Sym_Cnt++;
   }
  }
  Re_LO_Op=Im_LO_Op=0.0;
  Rx_Samp_Cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the current MF output.
*******************************************************************************/
int Compute_MF_Op_3(ip_cnt)
int ip_cnt;
{
 int cnt,inx;
 inx=Rx_Samp_Cnt-Tim_Track_Instant+TIM_WINDOW;
 if(inx < ZERO)
  Error_Mes(3);
 Re_MF_Op[inx]=Im_MF_Op[inx]=0.0;
 for(cnt=ip_cnt;cnt<MF_SIZE;cnt=cnt+RX_IP_FAC)
 {
  Re_MF_Op[inx]=Re_MF_Op[inx]+MF[cnt]*Re_MF_Reg[cnt];
  Im_MF_Op[inx]=Im_MF_Op[inx]+MF[cnt]*Im_MF_Reg[cnt];
 }
 if((Sym_Cnt%TWO) == ZERO)
  Store_MF_Op(inx);
 return(0);
}
/******************************************************************************/
/*            Store the MF o/p. Most recent sample is at ZERO.
*******************************************************************************/
int Store_MF_Op(inx)
int inx;
{
 Cmplx_Shift_from_Zero_2D(Re_MF_Op_Buf,Im_MF_Op_Buf,SYM_BUF_LEN_P1,inx);
 Re_MF_Op_Buf[inx][ZERO]=Re_MF_Op[inx];
 Im_MF_Op_Buf[inx][ZERO]=Im_MF_Op[inx];
 return(0);
}
/******************************************************************************/
