/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal10type.h"
#include "deal10ext.h"
/******************************************************************************/
/*                         Convert decimal to binary.
*******************************************************************************/
int Decimal_to_Binary(num,array,array_size)
int num,array[],array_size;
{
 int cnt;
 for(cnt=ZERO;cnt<array_size;cnt++)
  array[cnt]=ZERO;
 cnt=ZERO;
 while(num>ZERO)
 {
  if(cnt == array_size)
   Error_Mes(3);
  array[cnt]=num&1;
  num=num>>1;
  cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                    Convert binary array to decimal.
*******************************************************************************/
int Binary_to_Decimal(array,size)
int array[],size;
{
 int cnt,pow,sum;
 sum=ZERO;
 pow=1;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  sum=sum+pow*array[cnt];
  pow=pow<<1;
 }
 return(sum);
}
/******************************************************************************/
/* Real convolution.
   Input arrays: 'ip1[ip1size]' is first array.
                 'ip2[ip2size]' is second array.
   Output array: 'op[]'.
*******************************************************************************/
int Convolve(ip1,ip2,op,ip1size,ip2size)
int ip1size,ip2size;
double ip1[],ip2[],op[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  op[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    op[opcnt]=op[opcnt]+ip1[ipcnt]*ip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*             Copy a sequence in reverse complex conjugate order.
*******************************************************************************/
int Copy_Rev_Conjugate(re_orig,im_orig,re_copy,im_copy,size)
int size;
double re_orig[],im_orig[],re_copy[],im_copy[];
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  re_copy[cnt]=  re_orig[size-ONE-cnt];
  im_copy[cnt]= -im_orig[size-ONE-cnt];
 }
 return(0);
}
/******************************************************************************/
/* Complex convolution.
   Input arrays: 'xxip1[ip1size]' is first array.
                 'xxip2[ip2size]' is second array.
   Output array: 'xxop[]'.
*******************************************************************************/
int CConvolve(reip1,imip1,reip2,imip2,reop,imop,ip1size,ip2size)
int ip1size,ip2size;
double reip1[],imip1[],reip2[],imip2[],reop[],imop[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  reop[opcnt]=imop[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    reop[opcnt]=reop[opcnt]+reip1[ipcnt]*reip2[opcnt-ipcnt];
    reop[opcnt]=reop[opcnt]-imip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+reip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+imip1[ipcnt]*reip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*                      Get the root-raised cosine filter.
*******************************************************************************/
int Get_RRC(size,delay,samp_freq,array)
int size;
double delay,samp_freq,array[];
{
 int inx;
 double time;
 for(inx=ZERO;inx<size;inx++)
 {
  time=(double)inx-delay;
  if(fabs(time) > ERROR)
   Get_Term1(inx,time,samp_freq,array);
  else
   Get_Term2(inx,array);
 }
 return(0);
}
/******************************************************************************/
/*            Get the value which does not correspond to the peak.
*******************************************************************************/
int Get_Term1(inx,time,samp_freq,array)
int inx;
double time,samp_freq,array[];
{
 double theta1,theta2,numer,denom,term1,term2,term3,term4;
 time=time/samp_freq;
 theta1=M_PI*(1.0-ROLL_OFF)*time*SYMBOL_RATE;
 theta2=M_PI*(1.0+ROLL_OFF)*time*SYMBOL_RATE;
 term1=sin(theta1);
 term2=4.0*ROLL_OFF*time*SYMBOL_RATE*cos(theta2);
 numer=term1+term2;
 term3=4.0*ROLL_OFF*time*SYMBOL_RATE;
 term4=term3*term3;
 denom=M_PI*time*sqrt(SYMBOL_RATE)*(1.0-term4);
 if(fabs(denom) < ERROR)
  Error_Mes(1);
 array[inx]=numer/denom;
 return(0);
}
/******************************************************************************/
/*               Get the value which corresponds to the peak.
*******************************************************************************/
int Get_Term2(inx,array)
int inx;
double array[];
{
 array[inx]=sqrt(SYMBOL_RATE)*((1.0-ROLL_OFF)+4.0*ROLL_OFF/M_PI);
 return(0);
}
/******************************************************************************/
/*     Get the root-raised cosine filter in the presence of a clock error.
*******************************************************************************/
int Get_RRC_Clk_Err(size,delay,samp_freq,array,time_err)
int size;
double delay,samp_freq,array[],time_err;
{
 int inx;
 double time;
 for(inx=ZERO;inx<size;inx++)
 {
  time=(double)inx-delay;
  time=time/samp_freq+time_err;
  if(fabs(time) > ERROR)
   Get_Term1_Clk_Err(inx,time,array);
  else
   Get_Term2_Clk_Err(inx,array);
 }
 return(0);
}
/******************************************************************************/
/*            Get the value which does not correspond to the peak.
*******************************************************************************/
int Get_Term1_Clk_Err(inx,time,array)
int inx;
double time,array[];
{
 double theta1,theta2,numer,denom,term1,term2,term3,term4;
 theta1=M_PI*(1.0-ROLL_OFF)*time*SYMBOL_RATE;
 theta2=M_PI*(1.0+ROLL_OFF)*time*SYMBOL_RATE;
 term1=sin(theta1);
 term2=4.0*ROLL_OFF*time*SYMBOL_RATE*cos(theta2);
 numer=term1+term2;
 term3=4.0*ROLL_OFF*time*SYMBOL_RATE;
 term4=term3*term3;
 denom=M_PI*time*sqrt(SYMBOL_RATE)*(1.0-term4);
 if(fabs(denom) < ERROR)                             /* 0/0 term encountered. */
 {
  printf("\t Gen RRC: Tx samp:%d time:%18.14lf\n",Tx_Samp_Cnt,time);
  printf("\t Gen RRC: numer:%18.14lf denom:%18.14lf\n",numer,denom);
  fflush(stdout);
  Get_Term1_Clk_Err(inx,time+DITHER,array);                /* Recursive call. */
 }
 else
  array[inx]=numer/denom;
 return(0);
}
/******************************************************************************/
/*               Get the value which corresponds to the peak.
*******************************************************************************/
int Get_Term2_Clk_Err(inx,array)
int inx;
double array[];
{
 array[inx]=sqrt(SYMBOL_RATE)*((1.0-ROLL_OFF)+4.0*ROLL_OFF/M_PI);
 return(0);
}
/******************************************************************************/
/*                  Normalize the root raised cosine filter.
*******************************************************************************/
int Normalize_RRC(filter,size)
int size;
double filter[];
{
 int cnt;
 double energy,sqrt_energy;
 energy=0.0;
 for(cnt=ZERO;cnt<size;cnt++)
  energy=energy+filter[cnt]*filter[cnt];
 sqrt_energy=sqrt(energy);
 for(cnt=ZERO;cnt<size;cnt++)
  filter[cnt]=filter[cnt]/sqrt_energy;
 return(0);
}
/******************************************************************************/
/*         Shift the complex array away from zero at the transmitter.
*******************************************************************************/
int Cmplx_Shift_from_Zero_Tx(re_array,im_array,size,offset)
int size,offset;
double re_array[],im_array[];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
  re_array[cnt]=re_array[cnt-ONE];
 for(cnt=size+offset-ONE;cnt>ZERO;cnt--)
  im_array[cnt]=im_array[cnt-ONE];
 return(0);
}
/******************************************************************************/
/*                   Shift the complex array away from zero.
*******************************************************************************/
int Cmplx_Shift_from_Zero(re_array,im_array,size)
int size;
double re_array[],im_array[];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
 {
  re_array[cnt]=re_array[cnt-ONE];
  im_array[cnt]=im_array[cnt-ONE];
 }
 return(0);
}
/******************************************************************************/
/*                   Shift the complex array away towards zero.
*******************************************************************************/
int Cmplx_Shift_to_Zero(re_array,im_array,size)
int size;
double re_array[],im_array[];
{
 int cnt;
 for(cnt=ZERO;cnt<size-ONE;cnt++)
 {
  re_array[cnt]=re_array[cnt+ONE];
  im_array[cnt]=im_array[cnt+ONE];
 }
 return(0);
}
/******************************************************************************/
/*                  Shift the complex 2D array away from zero.
*******************************************************************************/
int Cmplx_Shift_from_Zero_2D(re_array,im_array,size,inx)
int size,inx;
double re_array[TIM_WIN_LEN][SYM_BUF_LEN_P1];
double im_array[TIM_WIN_LEN][SYM_BUF_LEN_P1];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
 {
  re_array[inx][cnt]=re_array[inx][cnt-ONE];
  im_array[inx][cnt]=im_array[inx][cnt-ONE];
 }
 return(0);
}
/******************************************************************************/
/*                   Shift the real array away from zero.
*******************************************************************************/
int Re_Shift_from_Zero(array,size)
int size;
double array[];
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
  array[cnt]=array[cnt-ONE];
 return(0);
}
/******************************************************************************/
/*                   Shift the real array towards zero.
*******************************************************************************/
int Re_Shift_to_Zero(array,size)
int size;
double array[];
{
 int cnt;
 for(cnt=ZERO;cnt<size-ONE;cnt++)
  array[cnt]=array[cnt+ONE];
 return(0);
}
/******************************************************************************/
/*                   Shift an integer array away from zero.
*******************************************************************************/
int Int_Shift_from_Zero(reg,size)
int reg[],size;
{
 int cnt;
 for(cnt=size-ONE;cnt>ZERO;cnt--)
  reg[cnt]=reg[cnt-ONE];
 return(0);
}
/******************************************************************************/
/*                   Shift an integer array towards zero.
*******************************************************************************/
int Int_Shift_to_Zero(reg,size)
int reg[],size;
{
 int cnt;
 for(cnt=ZERO;cnt<size-ONE;cnt++)
  reg[cnt]=reg[cnt+ONE];
 return(0);
}
/******************************************************************************/
/*            Get the avg. amplitude at the output of the IIR filter.
*******************************************************************************/
int Get_IIR_Avg_Amp(tim_amp,re_tmp,im_tmp,inx)
int inx;
double tim_amp,re_tmp,im_tmp;
{
 Avg_Re_Amp[inx]=TIIR_COEFF*Avg_Re_Amp[inx]+(1.0-TIIR_COEFF)*tim_amp;
 if(inx == TIM_WINDOW)
 {
  C_Avg_Re_Amp=CIIR_COEFF*C_Avg_Re_Amp+(1.0-CIIR_COEFF)*re_tmp;
  C_Avg_Im_Amp=CIIR_COEFF*C_Avg_Im_Amp+(1.0-CIIR_COEFF)*im_tmp;
 }
 return(0);
}
/******************************************************************************/
/*                        Normalize the alpha values.
*******************************************************************************/
int Normalize_Alpha_Beta(time)
int time;
{
 int state;
 double suma,sumb;
 suma=sumb=0.0;
 for(state=ZERO;state<NUM_STATES;state++)
 {
  suma=suma+Alpha[time+ONE][state];
  sumb=sumb+Beta[DATA_SIZE-ONE-time][state];
 }
 for(state=ZERO;state<NUM_STATES;state++)
 {
  Alpha[time+ONE][state]=Alpha[time+ONE][state]/suma;
  Beta[DATA_SIZE-ONE-time][state]=Beta[DATA_SIZE-ONE-time][state]/sumb;
 }
 return(0);
}
/******************************************************************************/
/*                    Print the alpha and beta values.
*******************************************************************************/
int Print_Alpha_Beta()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_STATES;cnt++)
 {
  printf("\t Iter:%d State:%d Alpha:%lf ",Iter,cnt,Alpha[PRINT_TIME][cnt]);
  printf(" Beta:%lf\n",Beta[PRINT_TIME+ONE][cnt]);
 }
 return(0);
}
/******************************************************************************/
/*  Convert uniform random variables to gaussian random variables. Refer to
 *  DSP by Proakis and Manolakis.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r < FRAC_ZERO)
  Error_Mes(5);
 r=sqrt(r);
 theta=2.0*M_PI*tnarry[UTHETA];
 tnarry[GRAND_0]=r*cos(theta);
 tnarry[GRAND_1]=r*sin(theta);
 return(0);
}
/******************************************************************************/
/*                             Error message.
*******************************************************************************/
int Error_Mes(num)
int num;
{
 fprintf(stdout,"\t Simulation error. Exiting with code %d.\n\n",num);
 fflush(stdout);
 exit(num);
 return(0);
}
/******************************************************************************/
