/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal4type.h"
#include "deal4ext.h"
/******************************************************************************/
int Get_ML_F_Off()
{
 Step_Size=F_Off_Iter=ZERO;
 F_Off_Found=FALSE;
 Prv_Est_F_Off=Est_F_Off_ML=Est_F_Off_2;
 Multiply_Sym();
 while((F_Off_Iter < MAX_F_OFF_ITER) && (F_Off_Found == FALSE))
 {
  Get_DFT();
  Get_Deriv();
  Update_ML_F_Off();
  F_Off_Iter++;
 }
 Update_ML_RMS_Err();
 Print_ML_Est();
 return(0);
}
/******************************************************************************/
/*            Multiply with the complex conjugate of the symbols.
*******************************************************************************/
int Multiply_Sym()
{
 int cnt,inx;
 double re_sym,im_sym,re_samp,im_samp;
 inx=Max_F_Off_Inx_2;
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  re_sym=Map[Sym[cnt]&ONE];
  im_sym=Im_ISI[cnt];
  re_samp=Re_MF_Op[inx];
  im_samp=Im_MF_Op[inx];
  Re_MF_Op[inx]=re_samp*re_sym+im_samp*im_sym;
  Im_MF_Op[inx]=im_samp*re_sym-re_samp*im_sym;
  inx=inx+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*                              Get the DFT.
*******************************************************************************/
int Get_DFT()
{
 int cnt,inx;
 double theta;
 inx=Max_F_Off_Inx_2;
 theta=Re_DFT=Im_DFT=0.0;
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  Re_DFT=Re_DFT+Re_MF_Op[inx]*cos(theta)+Im_MF_Op[inx]*sin(theta);
  Im_DFT=Im_DFT+Im_MF_Op[inx]*cos(theta)-Re_MF_Op[inx]*sin(theta);
  theta=theta+Est_F_Off_ML*(double)TX_IP_FAC;
  if(theta > 2.0*M_PI)
   theta=theta-2.0*M_PI;
  else if(theta < -2.0*M_PI)
   theta=theta+2.0*M_PI;
  inx=inx+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*                             Get the derivative.
*******************************************************************************/
int Get_Deriv()
{
 int cnt,inx;
 double theta,tmp;
 inx=Max_F_Off_Inx_2;
 theta=Re_Dv=Im_Dv=0.0;
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  Re_Dv=Re_Dv+(Re_MF_Op[inx]*cos(theta)+Im_MF_Op[inx]*sin(theta))*(double)cnt;
  Im_Dv=Im_Dv+(Re_MF_Op[inx]*sin(theta)-Im_MF_Op[inx]*cos(theta))*(double)cnt;
  theta=theta+Est_F_Off_ML*(double)TX_IP_FAC;
  if(theta > 2.0*M_PI)
   theta=theta-2.0*M_PI;
  else if(theta < -2.0*M_PI)
   theta=theta+2.0*M_PI;
  inx=inx+RX_SAMP_PER_SYM;
 }
 tmp=Re_Dv;
 Re_Dv= -Im_Dv;
 Im_Dv=  tmp;
 return(0);
}
/******************************************************************************/
/*                       Update the ML frequency estimate.
*******************************************************************************/
int Update_ML_F_Off()
{
 double grad,mag;
 grad=2.0*(Re_DFT*Re_Dv-Im_DFT*Im_Dv);
 mag=Re_DFT*Re_DFT+Im_DFT*Im_DFT;
 if(F_Off_Iter == ZERO)
  Prv_Grad=grad;
 if(grad*Prv_Grad > FRAC_ZERO)
  Proc_No_Transition(grad,mag);
 else
  Proc_Transition(grad,mag);
 return(0);
}
/******************************************************************************/
/*                      No change of sign in the gradient.
*******************************************************************************/
int Proc_No_Transition(grad,mag)
double grad,mag;
{
 double log_grad,t_exp,step;
 Prv_Est_F_Off=Est_F_Off_ML;
 Prv_Grad=grad;
 log_grad=log10(fabs(grad));
 printf("ML foff:%lf grad:%lf mag:%lf\n",Est_F_Off_ML,grad,mag);
 if(Step_Size == ZERO)
  t_exp= -3.0-rint(log_grad);
 else
  t_exp= -4.0-rint(log_grad);
 step=pow(10.0,t_exp);
 Est_F_Off_ML=Est_F_Off_ML+step*grad;
 return(0);
}
/******************************************************************************/
/*                  There is a change of sign in the gradient.
*******************************************************************************/
int Proc_Transition(grad,mag)
double grad,mag;
{
 double log_grad,t_exp,step;
 log_grad=log10(fabs(grad));
 printf("ML foff:%lf grad:%lf mag:%lf\n",Est_F_Off_ML,grad,mag);
 Step_Size++;
 if(Step_Size == TWO)
  F_Off_Found=TRUE;
 else
 {
  t_exp= -4.0-rint(log_grad);
  step=pow(10.0,t_exp);
  Est_F_Off_ML=Prv_Est_F_Off+step*grad;
 }
 return(0);
}
/******************************************************************************/
/*           Update the RMS error in the ML freq. offset estimate.
*******************************************************************************/
int Update_ML_RMS_Err()
{
 double diff;
 Est_F_Off_3=Prv_Est_F_Off;
 diff=fabs(F_OFF-Est_F_Off-Est_F_Off_3);
 F_Off_RMS_3=F_Off_RMS_3+diff*diff;
 if(diff > Max_F_Off_Err_3)
  Max_F_Off_Err_3=diff;
 return(0);
}
/******************************************************************************/
/*            Print the ML estimate of the residual freq. offset.
*******************************************************************************/
int Print_ML_Est()
{
 printf("\t Act. res. freq. offset:%lf\n",F_OFF-Est_F_Off);
 printf("\t ML. res. freq. offset:%lf\n",Prv_Est_F_Off);
 printf("\t ML. freq. offset err:%lf\n",F_OFF-Est_F_Off-Prv_Est_F_Off);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
