/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal4type.h"
#include "deal4ext.h"
/******************************************************************************/
int Simulation()
{
 for(Frame_Cnt=ZERO;Frame_Cnt<MAX_FRAMES;Frame_Cnt++)
 {
  Init_Frame();
  Get_Frame();                                            /* Go to deal4sym.c */
  Get_Tx_Flt();
  Get_Tx_Flt_Op();
  Get_F_Off();
  Receiver_2();
  Get_F_Off_2();
  Check_for_Frame_Erasure();
 }
 return(0);
}
/******************************************************************************/
/*                    Initialize the variables in a frame.
*******************************************************************************/
int Init_Frame()
{
 int cnt;
 double erand48();
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  Re_MF_Reg[cnt]=Im_MF_Reg[cnt]=FRAC_ZERO;
 if(RAND_TIM == ONE)
  Rand_Tx_Delay=(double)TX_DELAY+(double)TX_IP_FAC*erand48(N1_Store);
 else
  Rand_Tx_Delay=(double)TX_DELAY;
 if(RAND_PHASE == ONE)
  Initial_Phase=Theta=2.0*M_PI*erand48(N1_Store);
 else
  Initial_Phase=Theta=0.0*M_PI;
 Tx_Samp_Cnt=Rx_Samp_Cnt=ZERO;
 Tx_Reg_Init();
 return(0);
}
/******************************************************************************/
/*                        Get the transmit filter.
*******************************************************************************/
int Get_Tx_Flt()
{
 Get_RRC(TX_FLT_SIZE,Rand_Tx_Delay,TX_SAMP_FREQ,Tx_Flt); /* Go to deal4util.c */
 Normalize_RRC(Tx_Flt,TX_FLT_SIZE);
 return(0);
}
/******************************************************************************/
/*                       Get the transmit filter output.
*******************************************************************************/
int Get_Tx_Flt_Op()
{
 int cnt1,bit0,bit1,ip_cnt;
 double re_sym,im_sym;
 for(cnt1=ZERO;cnt1<FRAME_SIZE;cnt1++)
 {
  bit0=Sym[cnt1]&ONE;
  bit1=Sym[cnt1]>>ONE;
  re_sym=Map[bit0];
  im_sym=Map[bit1];
  for(ip_cnt=ZERO;ip_cnt<TX_IP_FAC;ip_cnt++)
  {
   Compute_Tx_Op(ip_cnt,re_sym,im_sym);
   re_sym=im_sym=0.0;
  }
 }
 return(0);
}
/******************************************************************************/
/*             Compute the current output of the transmit filter.
*******************************************************************************/
int Compute_Tx_Op(ip_cnt,re_sym,im_sym)
int ip_cnt;
double re_sym,im_sym;
{
 int cnt;
 Cmplx_Shift_from_Zero_Tx(Re_Tx_Reg,Im_Tx_Reg,TX_FLT_SIZE,TX_OFF);
 Re_Tx_Reg[ZERO]=re_sym;
 Im_Tx_Reg[ZERO]=im_sym;
 Re_Tx_Op=Im_Tx_Op=0.0;
 for(cnt=ip_cnt;cnt<TX_FLT_SIZE;cnt=cnt+TX_IP_FAC)
  Re_Tx_Op=Re_Tx_Op+Tx_Flt[cnt]*Re_Tx_Reg[cnt];
 for(cnt=ip_cnt;cnt<TX_FLT_SIZE_1;cnt=cnt+TX_IP_FAC)
 {
  if((cnt-TX_OFF) >= ZERO)
   Im_Tx_Op=Im_Tx_Op+Tx_Flt[cnt-TX_OFF]*Im_Tx_Reg[cnt];
 }
 Modulate();
 Send_to_Rx();
 return(0);
}
/******************************************************************************/
/*               Modulate the baseband signal and add noise.
*******************************************************************************/
int Modulate()
{
 double erand48();
 if(Tx_Samp_Cnt == TX_SAMP_SIZE)
  Error_Mes(3);
 if(Frame_Cnt == ZERO)
  fprintf(fp5,"%lf\n",sqrt(Re_Tx_Op*Re_Tx_Op+Im_Tx_Op*Im_Tx_Op));
 Passband_Signal[Tx_Samp_Cnt]=Re_Tx_Op*cos(Theta)-Im_Tx_Op*sin(Theta);
 N1_Val[URADIUS]=erand48(N1_Store);
 N1_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(Variance,N1_Val);                              /* Go to deal4util.c */
 Passband_Signal[Tx_Samp_Cnt]=Passband_Signal[Tx_Samp_Cnt]+N1_Val[GRAND_0];
 Theta=Theta+0.5*M_PI+F_OFF;
 if(Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 return(0);
}
/******************************************************************************/
/*                  Send the noisy passband signal to the receiver.
*******************************************************************************/
int Send_to_Rx()
{
 Demodulate_F_Off();
 Get_MF_Op();
 Tx_Samp_Cnt++;
 return(0);
}
/******************************************************************************/
/*                      Demodulate the passband signal.
*******************************************************************************/
int Demodulate_F_Off()
{
 Re_LO_Op=Passband_Signal[Tx_Samp_Cnt]*Re_LO[Tx_Samp_Cnt%LO_PERIOD];
 Im_LO_Op=Passband_Signal[Tx_Samp_Cnt]*Im_LO[Tx_Samp_Cnt%LO_PERIOD];
 return(0);
}
/******************************************************************************/
/*                            Get the MF output.
*******************************************************************************/
int Get_MF_Op()
{
 int cnt;
 for(cnt=ZERO;cnt<RX_IP_FAC;cnt++)
 {
  Cmplx_Shift_from_Zero(Re_MF_Reg,Im_MF_Reg,MF_SIZE);    /* Go to deal4util.c */
  Re_MF_Reg[ZERO]=Re_LO_Op;
  Im_MF_Reg[ZERO]=Im_LO_Op;
  Compute_MF_Op(cnt);
  Re_LO_Op=Im_LO_Op=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                       Compute the current MF output.
*******************************************************************************/
int Compute_MF_Op(ip_cnt)
int ip_cnt;
{
 int cnt;
 if(Rx_Samp_Cnt == RX_SAMP_SIZE)
  Error_Mes(3);
 Re_MF_Op[Rx_Samp_Cnt]=Im_MF_Op[Rx_Samp_Cnt]=0.0;
 for(cnt=ip_cnt;cnt<MF_SIZE;cnt=cnt+RX_IP_FAC)
 {
  Re_MF_Op[Rx_Samp_Cnt]=Re_MF_Op[Rx_Samp_Cnt]+MF[cnt]*Re_MF_Reg[cnt];
  Im_MF_Op[Rx_Samp_Cnt]=Im_MF_Op[Rx_Samp_Cnt]+MF[cnt]*Im_MF_Reg[cnt];
 }
 Rx_Samp_Cnt++;
 return(0);
}
/******************************************************************************/
/*                            Second iteration.
*******************************************************************************/
int Receiver_2()
{
 int cnt;
 Init_Rx_2();
 for(cnt=ZERO;cnt<TX_SAMP_SIZE;cnt++)
 {
  Demodulate_2(cnt);
  Get_MF_Op();
 }
 return(0);
}
/******************************************************************************/
/*                          Initialize receiver 2.
*******************************************************************************/
int Init_Rx_2()
{
 int cnt;
 C_Theta=FRAC_ZERO;
 Rx_Samp_Cnt=ZERO;
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  Re_MF_Reg[cnt]=Im_MF_Reg[cnt]=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
/*                     Second iteration of demodulation.
*******************************************************************************/
int Demodulate_2(cnt)
int cnt;
{
 Re_LO_Op=  2.0*Passband_Signal[cnt]*cos(C_Theta);
 Im_LO_Op= -2.0*Passband_Signal[cnt]*sin(C_Theta);
 C_Theta=C_Theta+0.5*M_PI+Est_F_Off;
 if(C_Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 return(0);
}
/******************************************************************************/
/*                         Check for frame erasure.
*******************************************************************************/
int Check_for_Frame_Erasure()
{
 if((Max_F_Off_Inx_2+(PRE_AMB_SIZE+SYM_SIZE-1)*RX_SAMP_PER_SYM)<RX_SAMP_SIZE)
 {
  Process_Nxt_Step();
  Sample_MF_Op();
 }
 else                                                       /* Frame erasure. */
  Erasure++;
 return(0);
}
/******************************************************************************/
/*                            Go to the nest step.
*******************************************************************************/
int Process_Nxt_Step()
{
 if(ATTEMPT_3 == TRUE)
 {
  Receiver_3();
  Get_F_Off_3();
 }
 else
  Get_ML_F_Off();
 return(0);
}
/******************************************************************************/
/*                            Second iteration.
*******************************************************************************/
int Receiver_3()
{
 int cnt;
 Init_Rx_3();
 for(cnt=ZERO;cnt<((TWO*PRE_AMB_SIZE)*TX_IP_FAC);cnt++)
 {
  Demodulate_3(cnt);
  Get_MF_Op();
 }
 return(0);
}
/******************************************************************************/
/*                          Initialize receiver 3.
*******************************************************************************/
int Init_Rx_3()
{
 int cnt;
 C_Theta=FRAC_ZERO;
 Rx_Samp_Cnt=ZERO;
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  Re_MF_Reg[cnt]=Im_MF_Reg[cnt]=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
/*                     Second iteration of demodulation.
*******************************************************************************/
int Demodulate_3(cnt)
int cnt;
{
 Re_LO_Op=  2.0*Passband_Signal[cnt]*cos(C_Theta);
 Im_LO_Op= -2.0*Passband_Signal[cnt]*sin(C_Theta);
 C_Theta=C_Theta+0.5*M_PI+Est_F_Off+Est_F_Off_2;
 if(C_Theta > (2.0*M_PI))
  Theta=Theta-2.0*M_PI;
 return(0);
}
/******************************************************************************/
