/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal4type.h"
#include "deal4ext.h"
/******************************************************************************/
/*                             Sample the MF output.
*******************************************************************************/
int Sample_MF_Op()
{
 F_Off_Cancel();
 Preamb_Ph_Proc();
 Init_ISI_Reg_for_Ph_Est();
 Data_Process();
 return(0);
}
/******************************************************************************/
/*                     Cancel the residual freq. offset.
*******************************************************************************/
int F_Off_Cancel()
{
 int cnt;
 double re_samp,im_samp;
 Est_Phase=FRAC_ZERO;
 Samp_Instant=Max_F_Off_Inx_2;
 for(cnt=ZERO;cnt<TWO*(PRE_AMB_SIZE+SYM_SIZE);cnt++)
 {
  if(Samp_Instant >= RX_SAMP_SIZE)
   Error_Mes(3);
  re_samp=Re_MF_Op[Samp_Instant];
  im_samp=Im_MF_Op[Samp_Instant];
  Re_MF_Op[Samp_Instant]=re_samp*cos(Est_Phase)+im_samp*sin(Est_Phase);
  Im_MF_Op[Samp_Instant]=im_samp*cos(Est_Phase)-re_samp*sin(Est_Phase);
  Samp_Instant=Samp_Instant+RX_OFF;
  Est_Phase=Est_Phase+Est_F_Off_3*(double)TX_OFF;
  if(Est_Phase > (2.0*M_PI))
   Est_Phase=Est_Phase-2.0*M_PI;
  else if(Est_Phase < (-2.0*M_PI))
   Est_Phase=Est_Phase+2.0*M_PI;
 }
 return(0);
}
/******************************************************************************/
/*                     Process the phase of the preamble.
*******************************************************************************/
int Preamb_Ph_Proc()
{
 int cnt;
 Samp_Instant=Max_F_Off_Inx_2;
 Avg_Re_Amp=Avg_Im_Amp=FRAC_ZERO;
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  Est_Pre_Amb_Phase(cnt);
  Samp_Instant=Samp_Instant+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*                      Estimate the preamble phase.
*******************************************************************************/
int Est_Pre_Amb_Phase(cnt)
int cnt;
{
 double re_sym,im_sym,mag,re_samp,im_samp;
 re_sym=Map[Sym[cnt]&ONE];
 im_sym=Im_ISI[cnt];
 mag=re_sym*re_sym+im_sym*im_sym;
 re_samp=Re_MF_Op[Samp_Instant]/mag;
 im_samp=Im_MF_Op[Samp_Instant]/mag;
 Avg_Re_Amp=IIR_COEFF*Avg_Re_Amp+(1.0-IIR_COEFF)*re_samp;
 Avg_Im_Amp=IIR_COEFF*Avg_Im_Amp+(1.0-IIR_COEFF)*im_samp;
// printf("Fr:%d sym%d ",Frame_Cnt,cnt);
// printf("re:%lf im:%lf ",re_samp,im_samp);
// printf("av re:%lf av im:%lf\n",Avg_Re_Amp,Avg_Im_Amp);
 return(0);
}
/******************************************************************************/
/* Initialize the ISI reg. for phase estimation. The most recent symbol is
 * at zero.
*******************************************************************************/
int Init_ISI_Reg_for_Ph_Est()
{
 int cnt;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
  ISI_Reg[cnt]=Map[Sym[PRE_AMB_SIZE-ONE-cnt]>>ONE];
 return(0);
}
/******************************************************************************/
/*                  Process the phase of the data symbols.
*******************************************************************************/
int Data_Process()
{
 int cnt;
 for(cnt=ZERO;cnt<(SYM_SIZE+SYM_BUF_LEN);cnt++)
 {
  Derotate(cnt);
  if(cnt >= SYM_BUF_LEN)
   Get_Errors(cnt);
  Shift_Into_ISI_Reg();
  Update_Data_Phase();
  Samp_Instant=Samp_Instant+RX_SAMP_PER_SYM;
 }
 return(0);
}
/******************************************************************************/
/*                  Derotate samples to get current symbol.
*******************************************************************************/
int Derotate(cnt)
int cnt;
{
 double re_tmp1,im_tmp1,re_tmp2,im_tmp2;
 if((Samp_Instant+RX_OFF) >= RX_SAMP_SIZE)
  Error_Mes(3);
 re_tmp1=Re_MF_Op[Samp_Instant];
 im_tmp1=Im_MF_Op[Samp_Instant];
 re_tmp2=Re_MF_Op[Samp_Instant+RX_OFF];
 im_tmp2=Im_MF_Op[Samp_Instant+RX_OFF];
 Est_Phase=atan(Avg_Im_Amp/Avg_Re_Amp);
 if(Avg_Re_Amp < FRAC_ZERO)
  Est_Phase=Est_Phase+M_PI;
 else if((Avg_Re_Amp > FRAC_ZERO) && (Avg_Im_Amp < FRAC_ZERO))
  Est_Phase=Est_Phase+2.0*M_PI;
 Cost=cos(Est_Phase);
 Sint=sin(Est_Phase);
 Rx_Re_Sym1=re_tmp1*Cost+im_tmp1*Sint;
 Rx_Im_Sym1=im_tmp1*Cost-re_tmp1*Sint;
 Rx_Re_Sym2=re_tmp2*Cost+im_tmp2*Sint;
 Rx_Im_Sym2=im_tmp2*Cost-re_tmp2*Sint;
// printf("ti:%d rxre:%lf rxim:%lf ",Samp_Instant,Rx_Re_Sym1,Rx_Im_Sym2);
// printf("txre:%d ",Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]&1);
// printf("txim:%d\n",Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]>>1);
 return(0);
}
/******************************************************************************/
/*                              Get the errors.
*******************************************************************************/
int Get_Errors(cnt)
int cnt;
{
 int bit0,bit1,rx_bit0,rx_bit1,cnt1;
 rx_bit0=rx_bit1=ZERO;
 bit0=Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]&ONE;
 bit1=Sym[cnt+PRE_AMB_SIZE-SYM_BUF_LEN]>>ONE;
 if(Rx_Re_Sym1 < FRAC_ZERO)
  rx_bit0=ONE;
 if(Rx_Im_Sym2 < FRAC_ZERO)
  rx_bit1=ONE;
 if(bit0 != rx_bit0)
  Ber++;
 if(bit1 != rx_bit1)
  Ber++;
 Valid_Bit=Valid_Bit+TWO;
 return(0);
}
/******************************************************************************/
/*  Shift new data into the ISI registers. The most recent symbol is at zero.
 *  The symbol is obtained two symbol instants from future time.
*******************************************************************************/
int Shift_Into_ISI_Reg()
{
 int cnt,inx;
 double re_tmp1,im_tmp1,re_sym3,im_sym3;
 for(cnt=ISI_LEN-ONE;cnt>ZERO;cnt--)
  ISI_Reg[cnt]=ISI_Reg[cnt-ONE];
 inx=Samp_Instant+SYM_BUF_LEN*RX_SAMP_PER_SYM+RX_OFF;
 if(inx >= RX_SAMP_SIZE)
  Error_Mes(3);
 re_tmp1=Re_MF_Op[inx];
 im_tmp1=Im_MF_Op[inx];
 re_sym3=re_tmp1*Cost+im_tmp1*Sint;
 im_sym3=im_tmp1*Cost-re_tmp1*Sint;
 if(im_sym3 > FRAC_ZERO)
  ISI_Reg[ZERO]=  1.0;
 else
  ISI_Reg[ZERO]= -1.0;
 return(0);
}
/******************************************************************************/
/*               Update the carrier phase of the data symbols.
*******************************************************************************/
int Update_Data_Phase()
{
 int cnt;
 double re_val,im_val,mag,re_tmp,im_tmp;
 im_val=0.0;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
  im_val=im_val+ISI_Coeff[cnt]*ISI_Reg[cnt];
 if(Rx_Re_Sym1 > FRAC_ZERO)
  re_val=  1.0;
 else
  re_val= -1.0;
 mag=re_val*re_val+im_val*im_val;
 re_tmp=(Re_MF_Op[Samp_Instant]*re_val+Im_MF_Op[Samp_Instant]*im_val)/mag;
 im_tmp=(Im_MF_Op[Samp_Instant]*re_val-Re_MF_Op[Samp_Instant]*im_val)/mag;
 Avg_Re_Amp=IIR_COEFF*Avg_Re_Amp+(1.0-IIR_COEFF)*re_tmp;
 Avg_Im_Amp=IIR_COEFF*Avg_Im_Amp+(1.0-IIR_COEFF)*im_tmp;
// printf("data Fr:%d ti:%d re sym:%lf ",Frame_Cnt,Samp_Instant,re_val);
// printf("imisi:%lf\n",im_val);
// printf("re:%lf im:%lf ",re_tmp,im_tmp);
// printf("avre:%lf avim:%lf\n",Avg_Re_Amp,Avg_Im_Amp);
 return(0);
}
/******************************************************************************/
