/******************************************************************************/
/*   Copyright (C) 2005 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                         PARAMETERS RELATED TO A FRAME.
*******************************************************************************/
#define PRE_AMB_SIZE 200                               /* Length of preamble. */
#define PRE_AMB_SS (PRE_AMB_SIZE-(ISI_LEN/TWO)+ONE) /* Steady-state preamble. */
#define DATA_SIZE 1000                                /* Length of data bits. */
#define DATA_SIZE_P1 (DATA_SIZE+ONE)                    /* Datasize plus one. */
#define SYM_SIZE (DATA_SIZE+DATA_SIZE/TWO)         /* Length of data symbols. */
#define POST_AMBLE 12                                      /* Postamble size. */
#define FRAME_SIZE (PRE_AMB_SIZE+SYM_SIZE+POST_AMBLE)           /* Framesize. */
#define MAX_FRAMES 1000                     /* No. of frames to be simulated. */
#define BPSK_SIZE 2                                    /* BPSK constellation. */
/******************************************************************************/
/*                         TRANSMIT FILTER PARAMETERS.
*******************************************************************************/
#define TX_DELAY 18                                 /* Transmit filter delay. */
#define TX_FLT_SIZE (TWO*TX_DELAY+ONE)                    /* Size of tx. flt. */
#define TX_OFF (TX_IP_FAC/TWO)                               /* Offset at tx. */
#define RX_OFF (TX_OFF*RX_IP_FAC)                            /* Offset at rx. */
#define TX_FLT_SIZE_1 (TX_FLT_SIZE+TX_OFF)            /* Size of im. tx. reg. */
#define SYMBOL_RATE 1.0                                       /* Symbol rate. */
#define TX_IP_FAC 6                            /* Interpolation factor at tx. */
#define TX_SAMP_FREQ ((double)TX_IP_FAC*SYMBOL_RATE)    /* Samp. freq. at tx. */
#define ROLL_OFF 0.41                                            /* Roll-off. */
/******************************************************************************/
/*                         FREQUENCY OFFSET PARAMETERS.
*******************************************************************************/
#define F_OFF (+0.000*M_PI)                       /* Freq. offset in radians. */
#define ISI_LEN 6                                /* Length of ISI in Im. arm. */
#define SYM_BUF_LEN ((ISI_LEN/2)-ONE)               /* Length of re. sym buf. */
#define ATTEMPT_3 FALSE        /* 1: 3rd attempt; 0: ML est. of freq. offset. */
#define MAX_F_OFF_ITER 20        /* No. of iter. for ML est. of freq. offset. */
#define MAX_F_OFF_ERR 0.05       /* Max. freq. off. est error in 2nd attempt. */
#define F_OFF_STEP 0.0005                  /* Step size of ML freq. off. est. */
#define IIR_COEFF 0.95                    /* IIR filter coeff. for phase avg. */
/******************************************************************************/
/*                         MATCHED FILTER PARAMETERS.
*******************************************************************************/
#define RX_IP_FAC 3                            /* Interpolation factor at rx. */
#define MF_DELAY (TX_DELAY*RX_IP_FAC)         /* Delay of the matched filter. */
#define MF_SIZE (TWO*MF_DELAY+ONE)                    /* Matched filter size. */
#define MF_AC_SIZE (TWO*MF_SIZE-ONE)                     /* MF autocor. size. */
#define RX_SAMP_FREQ (TX_SAMP_FREQ*(double)RX_IP_FAC)       /* MF samp. freq. */
#define RX_SAMP_SIZE (FRAME_SIZE*RX_SAMP_PER_SYM)     /* Rx. samp. per frame. */
#define TX_SAMP_SIZE (FRAME_SIZE*TX_IP_FAC)           /* Tx. samp. per frame. */
#define RX_SAMP_PER_SYM (TX_IP_FAC*RX_IP_FAC)         /* Rx. samples per sym. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define RAND_PHASE 1                        /* 1: Random phase; 0: otherwise. */
#define RAND_TIM 0                         /* 1: Random timing; 0: otherwise. */
#define SNR_B (+6.00)                                       /* SNR per bit. */
#define MAX_SNR 100.0
#define PN_SIZE 3                 /* Size of array which stores the PN value. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                        LOCAL OSCILLATOR PARAMETERS.
*******************************************************************************/
#define LO_PERIOD 4                                             /* LO period. */
/******************************************************************************/
/*                                CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define RE 0
#define IM 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MIN_MAG (-10.0)
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICE_PI 6.28318530717958647692
/******************************************************************************/
