/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal7type.h"
#include "deal7ext.h"
/******************************************************************************/
/*                   Initialize simulation variables.
*******************************************************************************/
int Sim_Init()
{
 Map_Init();
 Pre_Amble_Init();
 ISI_Init();
 Post_Amb_Init();
 Noise_Var_Init();
 Noise_Gen_Init();
 Iv_Noise_Gen_Init();
 LO_Init();
 Ber_Init();
 F_Off_Init();
 Amp_NVar_Err_Init();
 return(0);
}
/******************************************************************************/
/*                               BPSK mapping.
*******************************************************************************/
int Map_Init()
{
 Map[0]= 1.0;
 Map[1]=-1.0;
 return(0);
}
/******************************************************************************/
/*                       Initialize the preamble.
*******************************************************************************/
int Pre_Amble_Init()
{
 int cnt,bit0,bit1;
 long int random(void);
 printf("\t Preamble:\n");
 for(cnt=ZERO;cnt<PRE_AMB_SIZE;cnt++)
 {
  bit0=random()&ONE;
  bit1=random()&ONE;
  Sym[cnt]=TWO*bit1+bit0;
  printf("\t\t Time:%d re:%d im:%d\n",cnt,bit0,bit1);
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                 Initialize the intersymbol interference.
*******************************************************************************/
int ISI_Init()
{
 int cnt,im_bit[ISI_LEN];
 printf("\t Offset preamble:\n\n");
 Init_ISI_Reg();
 for(cnt=ZERO;cnt<PRE_AMB_SS;cnt++)
 {
  Get_Im_Bits(im_bit,cnt);
  Get_Im_Sym(im_bit,cnt);
  printf("\t\t Time:%d Re:%d Im:%lf\n",cnt,Sym[cnt]%TWO,Im_ISI[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*                         Initialize the ISI register.
*******************************************************************************/
int Init_ISI_Reg()
{
 int cnt;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
  ISI_Reg[cnt]=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
/*             Get the imaginary bits. Most recent bit is at zero.
*******************************************************************************/
int Get_Im_Bits(array,inx)
int array[],inx;
{
 int cnt,cnt1;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
 {
  cnt1=inx+cnt-ISI_LEN/TWO;
  if(cnt1 >= PRE_AMB_SIZE)
   Error_Mes(3);
  if((cnt1 >= ZERO) && (cnt1 < PRE_AMB_SIZE))
   array[ISI_LEN-ONE-cnt]=Sym[cnt1]>>ONE;
  else
   array[ISI_LEN-ONE-cnt]= MINUSONE;
 }
 return(0);
}
/******************************************************************************/
/*                          Get the imaginary part.
*******************************************************************************/
int Get_Im_Sym(array,inx)
int array[],inx;
{
 int cnt;
 double val;
 Im_ISI[inx]=FRAC_ZERO;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
 {
  if(array[cnt] == MINUSONE)
   val=FRAC_ZERO;
  else
   val=Map[array[cnt]];
  Im_ISI[inx]=Im_ISI[inx]+val*ISI_Coeff[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                            Initialize th postamble.
*******************************************************************************/
int Post_Amb_Init()
{
 int cnt,cnt1,bit0,bit1;
 long int random(void);
 cnt1=PRE_AMB_SIZE+SYM_SIZE;;
 for(cnt=ZERO;cnt<POST_AMBLE;cnt++)
 {
  bit0=random()&ONE;
  bit1=random()&ONE;
  if(cnt1 == FRAME_SIZE)
   Error_Mes(3);
  Sym[cnt1]=TWO*bit1+bit0;
  cnt1++;
 }
 return(0);
}
/******************************************************************************/
/*                   Initialize the transmit filter register.
*******************************************************************************/
int Tx_Reg_Init()
{
 int cnt;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Re_Tx_Reg[cnt]=0.0;
 for(cnt=ZERO;cnt<TX_FLT_SIZE_1;cnt++)
  Im_Tx_Reg[cnt]=0.0;
 return(0);
}
/******************************************************************************/
/*                        Initialize the noise variance.
*******************************************************************************/
int Noise_Var_Init()
{
 if(SNR_B < MAX_SNR)
 {
  Variance=1.0/pow(10.0,0.1*SNR_B);
  Inv_Twice_Noise_Var=1.0/(2.0*Variance);
 }
 else
  Variance=0.0;
 printf("\t SNR per bit:%lf Noise variance:%lf.\n",SNR_B,Variance);
 printf("\t Inv. twice noise variance:%lf.\n",Inv_Twice_Noise_Var);
 Variance=0.5*Variance;                /* Noise variance for passband signal. */
 printf("\t Variance of noise added to passband signal:%lf.\n\n",Variance);
 return(0);
}
/******************************************************************************/
/*                     Initialize the noise generator.
*******************************************************************************/
int Noise_Gen_Init()
{
 int cnt;
 long int random(void);
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
 {
  N1_Store[cnt]=(unsigned short)random();
  N2_Store[cnt]=(unsigned short)random();
 }
 return(0);
}
/******************************************************************************/
/*                Initialize the interleaver noise generator.
*******************************************************************************/
int Iv_Noise_Gen_Init()
{
 int cnt;
 Iv_Noise_Store[ZERO]=16882;
 Iv_Noise_Store[ONE]=7931;
 Iv_Noise_Store[TWO]=43491;
 for(cnt=ZERO;cnt<PN_SIZE;cnt++)
  printf("\t Iv Noise Index:%d Seed:%d\n",cnt,Iv_Noise_Store[cnt]);
 return(0);
}
/******************************************************************************/
/*                           Initialize the LO.
*******************************************************************************/
int LO_Init()
{
 int cnt;
 double theta1;
 printf("\t Local oscillator values:\n");
 for(cnt=ZERO;cnt<LO_PERIOD;cnt++)
 {
  theta1=2.0*M_PI*(double)cnt/(double)LO_PERIOD;
  Re_LO[cnt]=  rint(2.0*cos(theta1));
  Im_LO[cnt]= -rint(2.0*sin(theta1));
  printf("\t\t Time:%d cos:%+lf sin:%+lf\n",cnt,Re_LO[cnt],Im_LO[cnt]);
 }
 Tim_Var=Tim_Var_2=Tim_Var_3=0.0;
 return(0);
}
/******************************************************************************/
/*                           Initialize the BER.
*******************************************************************************/
int Ber_Init()
{
 Erasure=Valid_Bit=Ber=T_Ber=ZERO;
 return(0);
}
/******************************************************************************/
/*                   Initialize the freq. offset variables.
*******************************************************************************/
int F_Off_Init()
{
 Max_F_Off_Err=Max_F_Off_Err_2=Max_F_Off_Err_3=MIN_MAG;
 F_Off_RMS=F_Off_RMS_2=F_Off_RMS_3=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
/*   Initialize the amplitude and noise variance estimation error variables.
*******************************************************************************/
int Amp_NVar_Err_Init()
{
 Amp_RMS_Err=NVar_RMS_Err=FRAC_ZERO;
 return(0);
}
/******************************************************************************/
