/******************************************************************************/
/*   Copyright (C) 2006 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "deal7type.h"
#include "deal7ext.h"
/******************************************************************************/
int Pre_Processor()
{
 Open_Files();
 Get_Rx_MF();
 Sim_Init();                                             /* Go to deal7init.c */
 Get_Iv_Map();                                        /*  Random interleaver. */
 Ck_Var();                                                 /* Go to deal7ck.c */
 Get_Enc_Taps();
 Get_Trellis();
 return(0);
}
/******************************************************************************/
/*                                 Open files.
*******************************************************************************/
int Open_Files()
{
 fp1=fopen("deal7data/isi.dat","w");
 fp2=fopen("deal7data/foff.dat","w");
 fp3=fopen("deal7data/foff2.dat","w");
 fp4=fopen("deal7data/foff3.dat","w");
 fp5=fopen("deal7data/env.dat","w");
 if(fp1 == NULL)
  Error_Mes(4); 
 return(0);
}
/******************************************************************************/
/*                     Get the receiver matched filter.
*******************************************************************************/
int Get_Rx_MF()
{
 Get_RRC(MF_SIZE,(double)MF_DELAY,RX_SAMP_FREQ,MF);      /* Go to deal7util.c */
 Normalize_MF();
 Print_MF();
 Print_Autocorr();
 return(0);
}
/******************************************************************************/
/*                           Normalize the MF.
*******************************************************************************/
int Normalize_MF()
{
 int cnt;
 double energy,sqrt_energy,sqrt_ip_fac;
 sqrt_ip_fac=sqrt((double)RX_IP_FAC);
 energy=0.0;
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  energy=energy+MF[cnt]*MF[cnt];
 sqrt_energy=sqrt(energy);
 for(cnt=ZERO;cnt<MF_SIZE;cnt++)
  MF[cnt]=sqrt_ip_fac*MF[cnt]/sqrt_energy;
 return(0);
}
/******************************************************************************/
/*                         Print the matched filter.
*******************************************************************************/
int Print_MF()
{
 int inx;
 FILE *tfp;
 tfp=fopen("deal7data/mf.dat","w");
 for(inx=ZERO;inx<MF_SIZE;inx++)
  fprintf(tfp,"%lf\n",MF[inx]);
 fclose(tfp);
 return(0);
}
/******************************************************************************/
/*            Print the autocorrelation of the transmit filter.
*******************************************************************************/
int Print_Autocorr()
{
 int inx;
 double op[MF_AC_SIZE];
 FILE *tfp;
 tfp=fopen("deal7data/mfautocor.dat","w");
 Convolve(MF,MF,op,MF_SIZE,MF_SIZE);                     /* Go to deal7util.c */
 Get_ISI_Coeff(op);
 for(inx=ZERO;inx<MF_AC_SIZE;inx++)
  fprintf(tfp,"%lf\n",op[inx]);
 fclose(tfp);
 return(0);
}
/******************************************************************************/
/*                          Get the ISI coefficients.
*******************************************************************************/
int Get_ISI_Coeff(op)
double op[];
{
 int cnt,inx;
 for(cnt=ZERO;cnt<ISI_LEN;cnt++)
 {
  inx=MF_SIZE-ONE+RX_SAMP_PER_SYM/TWO+(cnt-ISI_LEN/TWO)*RX_SAMP_PER_SYM;
  ISI_Coeff[cnt]=op[inx]/(double)RX_IP_FAC;
  fprintf(fp1,"%lf %lf\n",(double)cnt,ISI_Coeff[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*                           Get encoder 1 taps.
*******************************************************************************/
int Get_Enc_Taps()
{
 Get_Enc_Numer_Taps();
 Get_Enc_Denom_Taps();
 return(0);
}
/******************************************************************************/
/*       Get encoder 1 numerator taps corresponding to 1+D^2+D^3+D^4.
*******************************************************************************/
int Get_Enc_Numer_Taps()
{
 Enc_N[0]=1;
 Enc_N[1]=0;
 Enc_N[2]=1;
 Enc_N[3]=1;
 Enc_N[4]=1;
 return(0);
}
/******************************************************************************/
/*         Get encoder 1 denominator taps corresponding to 1+D+D^4.
*******************************************************************************/
int Get_Enc_Denom_Taps()
{
 Enc_D[0]=1;
 Enc_D[1]=1;
 Enc_D[2]=0;
 Enc_D[3]=0;
 Enc_D[4]=1;
 return(0);
}
/******************************************************************************/
